from session import PROMPTS
PROMPT_MAIN = PROMPTS["NRP"]["MAIN"]
PROMPT_CONFIG = PROMPTS["NRP"]["CONFIG"]
PROMPT_LOGIN = PROMPTS["NRP"]["LOGIN"]
ALL_PROMPTS = PROMPTS["ALL"]["ALL"]
ALL_PROMPTS_MAIN = PROMPTS["ALL"]["MAIN"]
ALL_PROMPTS_CONFIG = PROMPTS["ALL"]["CONFIG"]

class Cli:
    """
    Class for CLI operations.
        
        Args:
            session (obj): Session object.

        Attributes:
            _s (session): An instance of the Session class.
    """

    def __init__(self, session: object):
        self._s = session

    def __chk_prompt(self, match_prompt: str):
        """
        Check if the current prompt is the same as match_prompt.

            Args:
                match_prompt (str): The prompt to be matched.

            Returns:
                bool: True if the current prompt is matched with match_prompt, False otherwise.
        """
        try_prompt = 2
        matched_prompt = 0
        for _ in range(try_prompt):
            ret = self._s.command_expect('', prompts=ALL_PROMPTS, timeout=2)
            print(ret)
            #check DUT is not in a logged out state
            if PROMPT_LOGIN in ret['data']:
                self._s.close()
                raise Exception(f"DUT logout!! Fail to back to main.\nret: {ret}")
            # print('ret["pattern"]: ', ret['pattern'])
            # print('type of ret["pattern"]: ', type(ret['pattern']))
            # print(isinstance(ret['pattern'], type))
            if ret['pattern'] == match_prompt or isinstance(
                    ret['pattern'], type):
                matched_prompt += 1
        return True if (matched_prompt > 0) else False

    def _back_to_main(self):
        """
        Exit until we get back to main prompt.

            Raises: 
                Exception: If exit too much.
        """
        self._s.clear_buffer()
        #try to get prompt        
        #check if already at main prompt
        rt = self.__chk_prompt(PROMPT_MAIN)
        if rt is True:
            return
        # exit until we get back to main prompt
        retry = 0
        retry_limit = 10
        while (rt is False) and (retry < retry_limit):
            rt = self._s.command_expect('exit',
                                        prompts=ALL_PROMPTS,
                                        timeout=2)
            if rt['pattern'] == PROMPT_MAIN:
                return
            rt = self.__chk_prompt(PROMPT_MAIN)
            if rt is False:
                retry += 1
        if (retry == retry_limit):
            raise Exception("exit too much.")

    def command(self, cmd: str, timeout: int = None):
        """
        Execute a command and return back to main prompt. 

            Args: 
                cmd (str): The command to be executed. 
                timeout (int, optional): Timeout for executing command in seconds. Defaults to None. 

            Raises: 
                ValueError: If cannot get next prompt in time when do command.  
        """
        ret = self._s.command_expect(cmd, timeout=timeout)
        if not ret['matched']:
            raise ValueError(
                f'cannot get next prompt in time when do command: {cmd}')
        return self

    def command_read(self,
                        cmd: str,
                        timeout: int = None,
                        parser: str = None):
        """
        Execute a command and read the output until the next prompt is matched.

            Args:
                cmd (str): The command to execute.
                timeout (int, optional): The maximum time in seconds to wait for the next prompt. Defaults to None.
                parser (str, optional): The parser to use when matching the next prompt. Defaults to None.
                
            Raises:
                ValueError: If the next prompt cannot be matched in time when executing the command. 
                
            Returns: 
                str: The output of the command execution.
        """
        ret = self._s.command_expect(cmd, parser=parser, timeout=timeout)
        if not ret['matched']:
            raise ValueError(
                f'cannot get next prompt in time when do command_read: {cmd}'
            )
        return ret['data']



import time

from behave import *
from steps import common, interface, login


@when(u'set ddns server with following table on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config().set_ddns(service = context.table[0]['service'], 
                                             name = context.table[0]['username'], 
                                             pwd = context.table[0]['password'],
                                             domain = context.table[0]['domain'])

@when(u'set dns server on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config_if_wan().set_dns(dns_server_1 = context.table[0]['dns_server_1'], 
                                                   dns_server_2 = context.table[0]['dns_server_2'],
                                                   dns_server_3 = context.table[0]['dns_server_3'])

@then(u'dig the domain name from "{host}" to check if the domain ip is replaced with "{device}"s {interface} ip')
def step_impl(context, host, device, interface):
    flag = False
    break_count = 0
    while not flag:
        ret_wan = context.dut[device].main().show_interface(name = 'wan')
        ret_dig = context.hosts[host].shellcmd.dig(context.table[0]['domain'])
        print(f'ret_wan: {ret_wan}, ret_dig: {ret_dig}')
        if ret_wan[0]['ip_address'].strip() in ret_dig:
            flag = True
        if break_count == 30:
            break
        break_count += 1
        time.sleep(10)
    assert flag, f'Domain ip is not changed by ddns, ori:{ret_dig}, expect:{ret_wan}'


import sys

from behave import *
from host import HostFactory


def loginToDUT(context, device):
    context.dut[device] = HostFactory.create_mdc_rp(
            context.dut_info[device], 
            debug_output=sys.stdout)
    return True


def logoutFromDUT(context, device):
    print(f"context.dut[{device}].disconnect()!!!!!")
    try:
        print(f"\n ---- dut log out start !!! ----")
        print(f"\n ---- dut back to main !!! ----")
        context.dut[device].main().exit()
        print(f"\n ---- dut close session !!! ----")
        context.dut[device].close()
        print(f"\n ---- dut log out finish!!! ----")
    except ValueError as e:
        print(f'\n ---- {e} ----')
        print(f'\n ---- dut is not alive ----')


@fixture
def loginToDUTWithFixture(context, device):
    try:
        # -- SETUP-FIXTURE
        loginToDUT(context, device)
        yield
    finally:
        # -- CLEANUP-FIXTURE
        logoutFromDUT(context, device)


@given('authorize CLI of "{device}"')
def step_impl(context, device):
    use_fixture(loginToDUTWithFixture, context, device)

@given('login "{device}" with ip "{ip}"')
def step_impl(context, device, ip):
    context.dut_info[device]['session']['host'] = ip
    loginToDUT(context, device)

@given('login "{device}" as Admin')
def step_impl(context, device):
    context.dut_info[device]['credential']['username'], context.dut_info[device]['credential']['password'] = 'admin', 'moxa'
    loginToDUT(context, device)


@given('logout "{device}"')
@when('logout "{device}"')
def step_impl(context, device):
    try:
        context.dut[device].main()._s.command('exit')
        context.dut[device].close()
    except ValueError as e:
        print(f'\n ---- {e} ----')


@then('login "{device}" as {user} with password {password} successfully')
def step_impl(context, device, user, password):
    context.dut_info[device]['credential']['username'], context.dut_info[device]['credential']['password'] = user, password

    try:
        context.dut[device] = HostFactory.create_mdc_rp(
            context.dut_info[device], 
            debug_output=sys.stdout)
    except ValueError as e:
        print(f'\n ---- {e} ----')
        assert False, f"The {user} should be able to login {device}"


@then('unable to login "{device}" as {user} with password {password}')
def step_impl(context, device, user, password):
    context.dut_info[device]['credential']['username'], context.dut_info[device]['credential']['password'] = user, password

    try:
        context.dut[device] = HostFactory.create_mdc_rp(
            context.dut_info[device], 
            debug_output=sys.stdout)
    except:
        print("OK! Suppose to catch exception!")
    else:
        assert False, f"The {user} should not be able to login {device}"



from session import PROMPTS
PROMPT_MAIN = PROMPTS["NRP"]["MAIN"]
PROMPT_CONFIG = PROMPTS["NRP"]["CONFIG"]
ALL_PROMPTS = PROMPTS["ALL"]["ALL"]
ALL_PROMPTS_MAIN = PROMPTS["ALL"]["MAIN"]
ALL_PROMPTS_CONFIG = PROMPTS["ALL"]["CONFIG"]

class Cli:


    def __init__(self, session):
        self._s = session

    def __chk_prompt(self, match_prompt):
        try_prompt = 2
        matched_prompt = 0
        for _ in range(try_prompt):
            ret = self._s.command_expect('', prompts=ALL_PROMPTS, timeout=2)
            # print(f'ret: {ret}')
            # print(f'match_prompt: {match_prompt}')
            # print(f"ret['pattern']: {ret['pattern']}")
            if ret['pattern'] == match_prompt or isinstance(
                    ret['pattern'], type):
                matched_prompt += 1
        return True if (matched_prompt > 0) else False

    def _back_to_main(self):
        self._s.clear_buffer()
        #try to get prompt
        #check DUT is not in a logged out state
        ret = self._s.command_expect('', prompts=['login as:'], timeout=2)
        if ret['matched']:
            self._s.close()
            raise Exception(f"DUT logout!! Fail to back to main.\nret: {ret}")
        #check if already at main prompt
        rt = self.__chk_prompt(PROMPT_MAIN)
        if rt is True:
            return
        # exit until we get back to main prompt
        retry = 0
        retry_limit = 10
        while (rt is False) and (retry < retry_limit):
            rt = self._s.command_expect('exit',
                                        prompts=ALL_PROMPTS,
                                        timeout=2)
            if rt['pattern'] == PROMPT_MAIN:
                return
            rt = self.__chk_prompt(PROMPT_MAIN)
            if rt is False:
                retry += 1
        if (retry == retry_limit):
            raise Exception("exit too much.")

    def command(self, cmd, timeout=None):
        ret = self._s.command_expect(cmd, timeout=timeout)
        if not ret['matched']:
            raise ValueError(
                f'cannot get next prompt in time when do command: {cmd}')
        return self

    def command_read(self, cmd, timeout=None, parser=None):
        ret = self._s.command_expect(cmd, parser=parser, timeout=timeout)
        if not ret['matched']:
            raise ValueError(
                f'cannot get next prompt in time when do command_read: {cmd}'
            )
        return ret['data']



from behave import *


@when('create IP Address and Subnet object with single IP on "{device}"')
def step_impl(context, device):
    context.obj_name = context.table[0]['Name']
    context.obj_addr = context.table[0]['IP Address']
    print(context.obj_name)
    context.dut[device].go_config_object_addr().create_object(
                                                name=context.obj_name,
                                                ip_addr=context.obj_addr)


@when('create IP Address and Subnet object with range IP on "{device}"')
def step_impl(context, device):
    context.obj_name = context.table[0]['Name']
    context.obj_addr = f"{context.table[0]['IP Address1']}-{context.table[0]['IP Address2']}"
    context.dut[device].go_config_object_addr().create_object(
                                                name=context.obj_name,
                                                ip_addr=context.obj_addr)


@when('create IP Address and Subnet object with subnet IP on "{device}"')
def step_impl(context, device):
    context.obj_name = context.table[0]['Name']
    context.obj_addr = f"{context.table[0]['Subnet IP']}/{context.table[0]['Subnet Mask']}"
    context.dut[device].go_config_object_addr().create_object(
                                                name=context.obj_name,
                                                ip_addr=context.obj_addr)


@then('the {IP_type} IP object is successfully created on "{device}"')
def step_impl(context, IP_type, device):
    ret = context.dut[device].main().show_object()
    flag = 0
    for i in ret:
        if i['obejct_name']==context.obj_name and i['detail']==context.obj_addr:
            flag = 1
            break
    assert flag, "the {IP_type} IP object is not created"


@when('create Network Service: "{service_type}" object with name: "{obj_name}" on "{device}"')
def step_impl(context, service_type, obj_name, device):
    context.service_type = service_type
    context.obj_name = obj_name
    context.dut[device].go_config_object_net_serv().create_object(
                                                name=obj_name,
                                                service=[service_type])


@then('the Network Service object is successfully created on "{device}"')
def step_impl(context, device):
    ret = context.dut[device].main().show_object()
    flag = 0
    for i in ret:
        if i['obejct_name']==context.obj_name and i['detail']==context.service_type:
            flag = 1
            break
    assert flag, f"the {context.service_type} object is not created"


@when('create Industrial Application Service: "{service_type}" object with name: "{obj_name}" on "{device}"')
def step_impl(context, service_type, obj_name, device):
    context.service_type = service_type
    context.obj_name = obj_name
    context.dut[device].go_config_object_industrial_app_serv().create_object(
                                                name=obj_name,
                                                service=[service_type])


@then('the Industrial Application Service object is successfully created on "{device}"')
def step_impl(context, device):
    ret = context.dut[device].main().show_object()
    flag = 0
    for i in ret:
        if i['obejct_name']==context.obj_name and i['detail']==context.service_type:
            flag = 1
            break
    assert flag, f"the {context.service_type} object is not created"


@when('create User-defined Service object with name: "{obj_name}" on "{device}"')
def step_impl(context, obj_name, device):
    context.obj_name = obj_name
    if context.table[0]['IP Protocol'] == "TCP" or context.table[0]['IP Protocol'] == "UDP":
        ''' Example:
        | IP Protocol | Service Port |
        | TCP         | any          |
        | TCP         | 22           |
        | TCP         | 22-23        |
        | UDP         | any          |
        | UDP         | 23           |
        | UDP         | 100-101      |
        '''
        if context.table[0]['Service Port']=='any':
            context.obj_detail = context.table[0]['IP Protocol']
        else:
            context.obj_detail = f"{context.table[0]['IP Protocol']} {context.table[0]['Service Port']}"
        context.dut[device].go_config_object_serv(context.table[0]['IP Protocol'].lower()).set_object_serv(
                                                name=obj_name,
                                                port=context.table[0]['Service Port'])
    elif context.table[0]['IP Protocol'] == "TCP and UDP":
        if context.table[0]['Service Port']=='any':
            context.obj_detail = "TCP; UDP"
        else:
            context.obj_detail = f"TCP {context.table[0]['Service Port']}; UDP {context.table[0]['Service Port']}"
        context.dut[device].go_config_object_serv("tcpudp").set_object_serv(
                                                name=obj_name,
                                                port=context.table[0]['Service Port'])
    elif context.table[0]['IP Protocol'] == "ICMP":
        if context.table[0]['ICMP type']=='any' and context.table[0]['ICMP code']=='any':
            context.obj_detail = "ICMP Type Any Code Any"
        else:
            context.obj_detail = f"ICMP Type {context.table[0]['ICMP type']} Code {context.table[0]['ICMP code']}"
        context.dut[device].go_config_object_serv("icmp").set_object_serv(
                                                name=obj_name,
                                                icmp_type=context.table[0]['ICMP type'],
                                                icmp_code=context.table[0]['ICMP code'])
    elif context.table[0]['IP Protocol'] == "Custom":
        context.obj_detail = f"IP Protocol {context.table[0]['Service Port']}"
        context.dut[device].go_config_object_serv("ipproto").set_object_serv(
                                                name=obj_name,
                                                ipproto=context.table[0]['Service Port'])


@then('the User-defined Service object is successfully created on "{device}"')
def step_impl(context, device):
    ret = context.dut[device].main().show_object()
    flag = 0
    for i in ret:
        if i['obejct_name']==context.obj_name and i['detail']==context.obj_detail:
            flag = 1
            break
    assert flag, f"the {context.obj_detail} object is not created"


from mdc.router.cli.rp_base.lg_router.tn5000 import config
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigDhcp(config.Config):
    """
    Class for configuring DHCP service.

        Args:
            session (object): The session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        # default apply the first element as parameter
        if str(self._args[0]).isdigit():
            index = self._args[0] if len(self._args) != 0 else None
            self._index = index
            if index is None:
                raise Exception(f"Input parameter is empty!")
            self._s.command_expect(f'ip dhcp pool {index}')
        else:
            name = self._args[0] if len(self._args) != 0 else None
            self._name = name
            if name is None:
                raise Exception(f"Input parameter is empty!")
            self._s.command_expect(f'ip dhcp static pool {name}')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_dhcp_server(self,
                        lease: str = None,
                        dns_server_1: str = None,
                        dns_server_2: str = None,
                        ntp_server: str = None,
                        default_router: str = None,
                        network_begin: str = None,
                        network_end: str = None,
                        network_mask: str = None,
                        host: str = None,
                        host_mask: str = None,
                        hardware_address: str = None):
        """
        Set dhcp server.

            Args:
                lease (str): The lease time of dhcp server
                dns_server_1 (str): The ip address of first dhcp server.
                dns_server_2 (str): The ip address of second dhcp server.
                ntp_server (str): The ip address of ntp server.
                default_router (str): The ip address of default router.
                network_begin (str): The begining ip address of given ip address pool.
                network_end (str): The end ip address of given ip address pool.
                network_mask (str): The netmask of given ip address pool.
                host (str): The ip address to assign to a host.
                host_mask (str): The netmask to assign to a host.
                hardware_address (str): The hardware address which is specified to assign ip address.
                
            Returns:
                self: The instance of the object. 
        """
        if network_begin and network_end and network_mask:
            # print('network')
            self._s.command_expect(
                f'network {network_begin} {network_end} {network_mask}')
        if host and host_mask:
            # print('host')
            self._s.command_expect(f'host {host} {host_mask}')
        if hardware_address:
            # print('mac')
            self._s.command_expect(f'hardware-address {hardware_address}')
        if default_router:
            # print('gw')
            self._s.command_expect(f'default-router {default_router}')
        if dns_server_1:
            # print('dns1')
            self._s.command_expect(
                f'dns-server {dns_server_1} {dns_server_2}')
        if ntp_server:
            # print('dns2')
            self._s.command_expect(f'ntp-server {ntp_server}')
        if lease:
            # print('lease')
            self._s.command_expect(f'lease {lease}')
        return self

    def exit(self):
        ret = self._s.command_expect('exit',
                                        prompts=[PROMPT_CONFIG],
                                        timeout=2)
        if ret['data'].find("You are setting") != -1:
            raise ValueError(f'Fail to set DHCP rule\n{ret["data"]}')



import os
import yaml


def prepare_json(context, file_name=None):
    try:
        lib_path = os.path.abspath(os.environ['PYTHONPATH'])
    except:
        raise Exception("PYTHONPATH not found!")
    abs_path = f'{lib_path}/topology'
    print("abspath:", abs_path)
    try:
        with open(f'{abs_path}/{file_name}.yaml') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
    except:
        print('topology can not be found, so use default instead')
        with open(f'{abs_path}/default.yaml') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)

    context.dut = {}
    context.hosts = {}
    context.dut_info = data["dut_info"]
    context.host_info = data["host_info"]
    context.generator_info = data["generator_info"]
    context.jig_info = data["jig_info"]



from mdc.router.cli.rp_base.ng_router import config, config_ipsec_phase1, config_ipsec_phase2
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigIpsec(config.Config):
    """
    Class for configuring IPSec.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        """
        Get into the interface lan.
        """
        # default apply the first element as parameter
        vpn_connection_name = self._args[0] if len(
            self._args) != 0 else None
        if vpn_connection_name is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'ipsec {vpn_connection_name}')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def set_vpn_connection(self,
                            remote_gateway: str = None,
                            interface: str = None,
                            startup_mode: str = None,
                            local_multi_network: str = None,
                            remote_multi_network: str = None,
                            dpd_action: str = None,
                            dpd_delay: str = None,
                            dpd_timeout: str = None,
                            identity: str = None):
        if remote_gateway:
            self._s.command_expect(f'remote-gateway {remote_gateway}')
        if interface:
            self._s.command_expect(f'interface {interface}')
        if startup_mode:
            self._s.command_expect(f'startup-mode {startup_mode}')
        if local_multi_network:
            self._s.command_expect(
                f'local-multi-network {local_multi_network}')
        if remote_multi_network:
            self._s.command_expect(
                f'remote-multi-network {remote_multi_network}')
        if identity:
            self._s.command_expect(f'identity {identity}')
        if dpd_action:
            self._s.command_expect(f'dpd-action {dpd_action}')
        if dpd_delay:
            self._s.command_expect(f'dpd-delay {dpd_delay}')
        if dpd_timeout:
            self._s.command_expect(f'dpd-timeout {dpd_timeout}')
        # quit to save
        self._s.command_expect(f'exit')
        return self

    def go_phase1(self, *args, **kwargs):
        """
        Configure an IPsec Phase 1.

            Args:
                *args: Variable length argument list.
                **kwargs: Arbitrary keyword arguments.

            Returns:
                object: An object of type rp_base_v2_0.ConfigIpsecPhase1.
        """
        return config_ipsec_phase1.ConfigIpsecPhase1(self._s, *args,
                                                **kwargs).getin()

    def go_phase2(self, *args, **kwargs):
        """
        Configure an IPsec Phase 2.

            Args:
                *args: Variable length argument list.
                **kwargs: Arbitrary keyword arguments.

            Returns:
                object: An object of type rp_base_v2_0.ConfigIpsecPhase2.
        """
        return config_ipsec_phase2.ConfigIpsecPhase2(self._s, *args,
                                                **kwargs).getin()



import re

from host import *
from behave import *
from steps import common, interface, login


@then('default system information of "{device}" is consistent with the spec')
def step_impl(context, device):
    sys_name = context.dut_info[device]['sys_name']
    sys_location = context.dut_info[device]['sys_location']
    sys_description = context.dut_info[device]['sys_description']

    ret = context.dut[device].main().show_system()
    cur_sys_name = re.findall(r'\D+', ret[0]['system_name'])[0].strip()

    assert cur_sys_name in sys_name, f'system name: {cur_sys_name} is not consistent with spec: {sys_name}'
    assert ret[0][
        'system_location'] in sys_location, f'sys location: {ret[0]["system_location"]} is not consistent with spec :{sys_location}'
    assert ret[0][
        'system_description'] in sys_description, f'sys description: {ret[0]["system_description"]} is not consistent with spec: {sys_description}'


@when('modify system information on "{device}"')
def step_impl(context, device):
    sys_name = context.table[0]['sys_name']
    sys_location = context.table[0]['sys_location']
    sys_description = context.table[0]['sys_description']
    context.dut[device].go_config().set_hostname(sys_name)
    context.dut[device].go_config().command(
        f'snmp-server location {sys_location}')
    context.dut[device].go_config().command(
        f'snmp-server description {sys_description}')


@then(
    'system information of "{device}" is consistent with the modified system information'
)
def step_impl(context, device):
    sys_name = context.table[0]['sys_name']
    sys_location = context.table[0]['sys_location']
    sys_description = context.table[0]['sys_description']

    ret = context.dut[device].main().show_system()

    assert ret[0][
        'system_name'] == sys_name, f'system name is not consistent with modified system information: {ret[0]["system_name"]}'
    assert ret[0][
        'system_location'] == sys_location, f'sys location is not consistent with modified system information :{ret[0]["system_location"]}'
    assert ret[0][
        'system_description'] == sys_description, f'sys description is not consistent with modified system information: {ret[0]["system_description"]}'



import os
import time

from behave import *

@given('enable static multicast route on "{device}"')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.MulticastRoute(cfg.get_session()).set_global_multicast_route(mode="static")


@given('set a static multicast route rule on "{device}"')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.MulticastRoute(cfg.get_session()).set_static_multicast_route_rule(action="add",
                                                                       group_addr=context.table[0]["dst_ip"],
                                                                       src_addr=context.table[0]["src_ip"],
                                                                       in_itf=context.table[0]["in_iface"],
                                                                       out_itf=context.table[0]["out_iface"])
    context.dut[device].main().save()


@when('"{host_send}" send {count} multicast packet with destination ip "{dst_ip}"')
def step_impl(context, host_send, count, dst_ip):
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host_send]['session']['credential']['password'],
                                                            "../lib/replay_pkts/multicast.pcap",
                                                            context.host_info[host_send]['session']['credential']['username'],
                                                            context.host_info[host_send]['session']['host']))

    context.hosts[host_send].tcp_tool.rewrite(infile="multicast.pcap", outfile="multicast_out.pcap",
                                                sip=context.host_info[host_send]["testbed"]["cur_host"], 
                                                dip=dst_ip, 
                                                smac=context.host_info[host_send]["testbed"]["mac_address"])
    context.hosts[host_send].tcp_tool.send(sendfile="multicast_out.pcap", number=count)
    context.multicast_group_addr = dst_ip
    time.sleep(10)


@then('"{host_recv}" {action} receive multicast packets from "{host_send}"')
def step_impl(context, host_recv, action, host_send):
    '''
    action can be 'shall' or 'shall not'
    '''
    context.hosts[host_recv].tshark.stop()

    ret = context.hosts[host_recv].tshark.retrieve()
    print('receive packets: ', ret)

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    ret_lines = ret.split('\n')

    destip_count = {}
    # parse and get dest IP counts
    for a in ret_lines:
        # skip if not UDP
        if a.find("UDP") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        if arr[4] in destip_count:
            destip_count[arr[4]] += 1
        else:
            destip_count[arr[4]] = 1

    # compare dest ip count
    host_recv_ip = context.multicast_group_addr

    if host_recv_ip not in destip_count:
        recv_packet_count = 0
    else:
        recv_packet_count = destip_count[host_recv_ip]

    if action == "shall":
        assert int(recv_packet_count) > 0, f'{host_recv} expect to receive packets but receive {recv_packet_count}'
    elif action == "shall not":
        assert int(recv_packet_count) == 0, f'{host_recv} expect to not receive packets but receive {recv_packet_count}'


from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigSessionControl(config.Config):
    """
        Class for configuring Session Control.

            ******** TBD ********
            Args:
                session (object): A valid session object.
                *args: Variable length argument list.
                **kwargs: Arbitrary keyword arguments.

            Attributes:
                _s (Session): A Session object used to communicate with the device.
                _args (list): A list of arguments passed to the function.
                _kwargs (dict): A dictionary of keyword arguments passed to the function.

            ******** /TBD ********
        """

    def __init__(self, session: object, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
            Set the default prompts.
            """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        """
            Get into the session control.
            """
        self._s.command_expect(f'session-control')

    def getin(self):
        """
            Set the default prompts and get into the session control.

                Returns: 
                    self: The instance of the object. 
            """
        self._set_default_prompts()
        self._getin()
        return self

    def set_session_ctrl(self,
                         name: str = None,
                         enable: bool = None,
                         disable: str = None,
                         logging: str = None,
                         policy_action: str = None,
                         dip: str = None,
                         dport: str = None,
                         ttl_tcp_conn: str = None,
                         concur_tcp_conn: str = None):
        """
            Set Session Control policy on the device.

                Args:
                    name (str): Policy Name. 
                    enable (bool): Enable the Policy. 
                    disable (str): Disable the policy. 
                    logging (str): Enable or disable logging for this policy. 
                    policy_action (str): Allow or deny action for this policy. 
                    dip (str): Matching IP Address and Subnet Object as Destination IP. 
                    dport (str): Matching User-defined Service Object as Destination Port. 
                    ttl_tcp_conn(str):  Total TCP Connections Limitation.
                    concur_tcp_conn(str): Concurrent TCP Requests Limitation.

                Returns:   
                    self: The instance of the object.   
            """
        if name:
            self._s.command_expect(f'name {name}')
        if enable:
            self._s.command_expect('enable')
        if logging == "enable":
            self._s.command_expect('logging flash')  # left to be complete
        if policy_action in ["allow", "deny"]:
            if policy_action == 'allow':
                self._s.command_expect(f"action {'monitor'}")
            else:
                self._s.command_expect(f"action {'drop'}")
        if dip:
            self._s.command_expect(f"dst-ip {dip}")
        if dport:
            self._s.command_expect(f"dst-port {dport}")
        if ttl_tcp_conn:
            self._s.command_expect(f"total-tcp-conn {int(ttl_tcp_conn)}")
        if concur_tcp_conn:
            self._s.command_expect(
                f"concurrent-tcp-conn {int(concur_tcp_conn)}")
        if disable in ["no"]:
            pass  # left to be complete
        return self



import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login


@given(u'enable dhcp service on "{device}"')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.DhcpService(cfg.get_session()).set_global_dhcp_service("enable")


@given(
    u'set configuration of {mode} dhcp server on "{device}" for dispatching ip to "{host}"'
)
def step_impl(context, mode, device, host):
    cfg = context.dut[device].go_config()
    if mode == "dhcp":
        context.host_info[list(context.hosts.keys(
        ))[0]]['testbed']['cur_host'] = context.table[0]['pool_ip_begin']
        context.dut[device].go_config_dhcp(1).set_dhcp_server(\
            network_begin=context.table[0]['pool_ip_begin'], \
            network_end=context.table[0]['pool_ip_end'], \
            network_mask=context.table[0]['mask'], \
            lease = context.table[0]['lease_time'], \
            default_router=context.table[0]['default_gw']). \
        exit()
        cfg.DhcpService(cfg.get_session()).set_global_dhcp_server(
            mode="pool", action="enable", index="1")

    elif mode == "mac-based":
        context.host_info[host]['testbed']['cur_host'] = context.table[1][
            'ip_host']
        context.dut[device].go_config_dhcp(mode).set_dhcp_server(\
            host=context.table[1]['ip_host'], \
            host_mask=context.table[1]['mask'], \
            hardware_address=context.host_info[host]['testbed']['mac_address'], \
            lease=context.table[1]['lease_time'], \
            default_router=context.table[1]['default_gw']). \
        exit()
        cfg.DhcpService(cfg.get_session()).set_global_dhcp_server(
            mode="static pool", action="enable", name=mode)
    else:
        raise ValueError(f'input dhcp mode is incorrect: {mode}')


@when(u'enable dhcp client in "{host}"')
def step_impl(context, host):
    context.hosts[host].shellcmd.enable_dhcp_client(
        dev=context.host_info[host]['nic'])


@then(u'"{host}" can receive ip address from {mode} server')
def step_impl(context, host, mode):
    count = 0
    ret = '' \
    if context.hosts[host].shellcmd.show_network(dev=context.host_info[host]['nic']) is None \
    else context.hosts[host].shellcmd.show_network(dev=context.host_info[host]['nic'])
    while 'inet' not in ret and count != 10:
        time.sleep(1)
        ret = '' \
        if context.hosts[host].shellcmd.show_network(dev=context.host_info[host]['nic']) is None \
        else context.hosts[host].shellcmd.show_network(dev=context.host_info[host]['nic'])
        count += 1
    print('ret:', ret)
    host_ip = re.findall(r'\d+\.\d+\.\d+\.\d+', ret)[0]
    cur_ip = host_ip.split('.')[-1]
    begin_ip = context.table[0]['pool_ip_begin'].split('.')[-1]
    end_ip = context.table[0]['pool_ip_end'].split('.')[-1]
    context.host_info[host]['testbed']['cur_host'] = host_ip
    if mode == "dhcp":
        assert int(begin_ip) <= int(cur_ip) <= int(
            end_ip
        ), f'{host} can not get ip within given pool from dhcp server: {host_ip}'
    elif mode == "mac-based":
        assert host_ip == context.table[0][
            'ip_host'], f'{host} can not get specified ip from dhcp server: {host_ip}'
    else:
        raise ValueError(f'input mode is incorrect: {mode}')


@then(u'"{device}" can ping "{host}"s received ip address')
def step_impl(context, device, host):
    ret = context.dut[device].main().ping(
        context.host_info[host]['testbed']['cur_host'])
    assert int(
        ret['received']) > 0, f'{device} can not ping {host} successfully'



import time
import random
from behave import *
from steps import common, interface, login

@when(u'send {pkt_count} arp packet from "{host}" to "{device}"')
def step_impl(context, pkt_count, host, device):
    target_ip = context.dut_info[device]["testbed"]["lan_ip"]
    host_ip = context.host_info[host]["testbed"]["cur_host"]
    connected_port = '1/' + context.host_info[host]["testbed"]["port"]
    vlan_res = context.dut[device].main().show_vlan()
    source_macs = []

    for _ in range(int(pkt_count)):
        mac = [0x00, 0x16, 0x3e,
            random.randint(0x00, 0x7f),
            random.randint(0x00, 0xff),
            random.randint(0x00, 0xff)]
        mac_address = ":".join(f"{x:02x}" for x in mac)
        source_macs.append(mac_address)
    ips = set()
    while len(ips) < int(pkt_count):
        ips.add(f"192.168.{random.randint(0, 255)}.{random.randint(1, 254)}")
    ips = list(ips)

    # fill arp scapy script with args
    script = context.hosts[host].scapy.generate_arp_scapy_script(source_macs, ips, target_ip, context.host_info[host]["nic"])
    from datetime import datetime
    import os

    # write the scapy script to file
    scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
    with open(f'../lib/atb/service/scapy_template/{scapy_file_name}', "w") as script_file:
        script_content = script_file.write(script)

    # scp the file to remote host with scapy service
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host]['session']['credential']['password'],
                                                            f"../lib/atb/service/scapy_template/{scapy_file_name}",
                                                            context.host_info[host]['session']['credential']['username'],
                                                            context.host_info[host]['session']['host']))
    
    context.hosts[host].scapy.do_script(scapy_file_name)
    # Clean up
    os.system(f'rm ../lib/atb/service/scapy_template/{scapy_file_name}')
    context.arp_info = {
        "mac": context.host_info[host]["testbed"]["mac_address"],
        "ip" : ips
    }

    for entry in vlan_res:
        if connected_port in entry['acs_ports'] or entry['trk_ports'] or \
                             entry['hyb_ports'] or entry['brg_ports']:
            vlan = entry['vid']
    
    context.mac_info = {
        "mac": source_macs,
        "vlan": vlan,
        "port": f'1/{context.host_info[host]["testbed"]["port"]}'
    }


@then(u'arp entry should be cleared within the aging time on "{device}"')
def step_impl(context, device):
    ret = context.dut[device].main()._s.command_expect("show arp")
    count = ret["data"].count(context.arp_info["mac"])
    aging_time = 0
    while count >= int(context.table[0]["remove_thershold"]) and \
        aging_time <= int(context.table[0]["aging_time"]):
        time.sleep(10)
        ret = context.dut[device].main()._s.command_expect('show arp')
        count = ret["data"].count(context.arp_info["mac"])
        aging_time += 10

    assert count < int(context.table[0]["remove_thershold"]), \
        f'arp table could not be cleared within the aging time: {ret["data"]}'
    
@then(u'arp entry should be recorded on "{device}"')
def step_impl(context, device):
    arp_table = context.dut[device].main().show_arp()
    for ip in context.arp_info["ip"]:
        flag = False
        for entry in arp_table:
            if ip in entry["address"] and \
                context.arp_info["mac"] in entry["hardware_addr"]:
                flag = True
                break
        assert flag, \
            f'expect-> [mac: {context.arp_info["mac"]}, \
                ip: {ip}\n but {arp_table}'


import time

from host import *
from behave import *
from steps import login, interface, common
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000


@then(
    u'check if "{host}" {can_or_cannot} receive traffic with "{device}"s {interface} ip and {specific_protocols}'
)
def step_impl(context, host, can_or_cannot, specific_protocols, device,
              interface):
    # ['ICMP' , 'Flags [S]', 'UDP']
    flag = { \
        'can': True, \
        'cannot': False}[can_or_cannot]
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve()
    print(ret)
    count = 0
    if ret != None:
        print(
            f'{context.dut_info[device]["testbed"][f"{interface.lower()}_ip"]} → {context.host_info[host]["testbed"]["cur_host"]} {specific_protocols.upper()}'
        )
        count = ret.count(
            f'{context.dut_info[device]["testbed"][f"{interface.lower()}_ip"]} → {context.host_info[host]["testbed"]["cur_host"]} {specific_protocols.upper()}'
        )
    if flag:
        assert count >= 1, f'expect receive 4 number of {specific_protocols.upper()} but {count}'
    else:
        assert count == 0, f'expect receive 0 number of {specific_protocols.upper()} but {count}'


@given('"{device}" insert {mode} nat rules')
@given('set {mode} nat rule on "{device}"')
def step_impl(context, device, mode):
    for rule in context.table:
        index_arr = rule['nat_indexes'].split("-")
        index_start = int(index_arr[0])
        index_end = (int(index_arr[0]) +
                     1) if len(index_arr) == 1 else (int(index_arr[1]) + 1)
    if issubclass(context.dut[device]._model, tn5000):
        if mode == '1-1':
            context.dut[device].go_config_nat().set_one2one_rule(ipaddr_in=rule['real_dest_ip'], \
                                                                 ipaddr_out=rule['nat_dest_ip'], \
                                                                 if_out=rule['nat_iface'])
        elif mode == 'N-1':
            context.dut[device].go_config_nat().set_dynamic_rule(ipaddr_start=rule['source_ip_start'], \
                                                                 ipaddr_end=rule['source_ip_end'], \
                                                                    if_out=rule['outgoing_interface'])
        elif mode == 'PAT':
            context.dut[device].go_config_nat().set_port_forward_rule(ipaddr_in=rule['real_dest_ip'], \
                                                                      port_in=rule['real_dest_port'], \
                                                                      if_out=rule['nat_iface'], \
                                                                      port_out=rule['nat_dest_port'], \
                                                                      protocol=rule['protocol'].lower())
        else:
            raise ValueError('input invalid mode of NAT')
    elif issubclass(context.dut[device]._model, Base):
        for rule in context.table:
            index_arr = rule['nat_indexes'].split("-")
            index_start = int(index_arr[0])
            index_end = (int(index_arr[0]) +
                        1) if len(index_arr) == 1 else (int(index_arr[1]) + 1)
            if mode == '1-1':
                for idx in range(index_start, index_end):
                    context.dut[device].go_config_nat(idx).\
                        mode(mode).\
                        original(
                            in_iface=rule["nat_iface"],
                            osi="any",
                            osp="any",
                            odi=rule["nat_dest_ip"],
                            odp="any").\
                        translated(
                            out_iface="any",
                            tsi="any",
                            tsp="any",
                            tdi=rule["real_dest_ip"],
                            tdp="any"
                        ).\
                        exit()
            elif mode == 'N-1':
                for idx in range(index_start, index_end):
                    context.dut[device].go_config_nat(idx).\
                        mode("n-1").\
                        original(
                            in_iface="any",
                            osi=f'{rule["source_ip_start"]}-{rule["source_ip_end"]}',
                            osp="any",
                            odi="any",
                            odp="any").\
                        translated(
                            out_iface=rule["outgoing_interface"],
                            tsi="any",
                            tsp="any",
                            tdi="any",
                            tdp="any"
                        ).\
                        exit()
            elif mode == 'PAT':
                for idx in range(index_start, index_end):
                    context.dut[device].go_config_nat(1).\
                        mode("pat").\
                        original(
                            in_iface=rule['nat_iface'],
                            osi="any",
                            osp="any",
                            odi="any",
                            odp=rule['nat_dest_port']).\
                        translated(
                            out_iface="any",
                            tsi="any",
                            tsp="any",
                            tdi=rule["real_dest_ip"],
                            tdp=rule['real_dest_port']
                        ).\
                        protocol(rule["protocol"].lower()).\
                        exit()
        

        # Wait 5 secs for rule to activate.
        time.sleep(5)


@when('"{host}" send some icmp echo request packets for each destination IP')
def step_impl(context, host):
    context.hosts[host].shellcmd.restart_network(context.host_info[host]["nic"])
    for item in context.table:
        context.hosts[host].hping3.send( \
            tcpudp_flags=['icmp'], \
            host = item['nat_dest_ip'], \
            count = item['pkt_num'],
            interval = "100000")
        time.sleep(2)


@then(
    'check "{host}" receive some icmp echo request packets with destination IP')
def step_impl(context, host):
    context.hosts[host].tshark.stop()

    ret = context.hosts[host].tshark.retrieve()

    ret_lines = ret.split('\n')

    destip_count = {}
    # parse and get dest IP counts
    for a in ret_lines:
        # skip if not ICMP request
        if a.find("Echo (ping) request") == -1:
            continue

        arr = a.lstrip(' ').split(' ')

        # skip unexpected format
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        if arr[4] in destip_count:
            destip_count[arr[4]] += 1
        else:
            destip_count[arr[4]] = 1

    # compare dest ip count
    for item in context.table:
        assert item[
            'real_dest_ip'] in destip_count, f'{item[f"real_dest_ip"]} expect to receive {item[f"pkt_num"]} packets but receive 0'

        assert int(destip_count[item['real_dest_ip']]) == int(
            item['pkt_num']
        ), f'{item[f"real_dest_ip"]} expect to receive {item["pkt_num"]} packets but receive {destip_count[item["real_dest_ip"]]}'



from mdc.router.cli.rp_base.lg_router.tn5000 import config, config_ipsec_phase1, config_ipsec_phase2
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigIpsec(config.Config):
    """
    Class for configuring IPSec.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    # Reserve flexibility for function override
    def _getin(self):
        # default apply the first element as parameter
        vpn_connection_name = self._args[0] if len(
            self._args) != 0 else None
        if vpn_connection_name is None:
            raise Exception(f"Input parameter is empty!")
        self._s.command_expect(f'ipsec {vpn_connection_name}')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def go_phase1(self, *args, **kwargs):
        return config_ipsec_phase1.ConfigIpsecPhase1(
            self._s, *args, **kwargs).getin()

    def go_phase2(self, *args, **kwargs):
        return config_ipsec_phase2.ConfigIpsecPhase2(
            self._s, *args, **kwargs).getin()



import re
import time
import random
from behave import *
from steps import common, interface, login

@when(u'send {pkt_count} rstp packet with the content of the table from "{host}" to "{device}"')
def step_impl(context, pkt_count, host, device):
    padding_n = 8
    dmac = "01:80:c2:00:00:00"
    brid = context.table[0]['brid']
    max_age = context.table[0]['max_age']
    iface = context.host_info[host]['nic']
    path_cost = context.table[0]['path_cost']
    fwd_delay = context.table[0]['fwd_delay']
    hello_time = context.table[0]['hello_time']
    smac = context.host_info[host]['testbed']['mac_address']
    script = context.hosts[host].scapy.generate_rstp_scapy_script(smac=smac,
                                                                  dmac=dmac,
                                                                  brid=brid,
                                                                  iface=iface,
                                                                  max_age=max_age,
                                                                  path_cost=path_cost, 
                                                                  fwd_delay=fwd_delay,
                                                                  hello_time=hello_time,
                                                                  padding_n = padding_n)
    from datetime import datetime
    import os

    # write the scapy script to file
    scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
    with open(f'../lib/atb/service/scapy_template/{scapy_file_name}', "w") as script_file:
        script_content = script_file.write(script)
    
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host]['session']['credential']['password'],
                                                            f"../lib/atb/service/scapy_template/{scapy_file_name}",
                                                            context.host_info[host]['session']['credential']['username'],
                                                            context.host_info[host]['session']['host']))
    # Clean up
    for _ in range(int(pkt_count)):
        context.hosts[host].scapy.do_script(scapy_file_name)
    context.hosts[host].scapy.clean_up()
    os.system(f'rm ../lib/atb/service/scapy_template/{scapy_file_name}')

@when(u'{action} spanning tree on "{device}" with following table')
def step_impl(context, device, action):
    priority = context.table[0]["brid"]
    hello_time = context.table[0]["hello_time"]
    fwd_delay = context.table[0]["fwd_delay"]
    max_age = context.table[0]["max_age"]
    context.dut[device].go_config_rstp().set_rstp_rule(priority,
                                                       hello_time,
                                                       fwd_delay,
                                                       max_age)
    cfg = context.dut[device].go_config()
    cfg.Rstp(cfg.get_session()).set_rstp_status(action)

@when(u'{action} spanning tree on port {port} of "{device}" with following table')
def step_impl(context, action, port, device):
    edge_port = True if context.table[0]["edge_port"] == "enable" else False
    priority = context.table[0]["pid"]
    cost = context.table[0]["path_cost"]
    cfg = context.dut[device].go_config()
    cfg.Rstp(cfg.get_session()).set_rstp_port(port,
                                              action = action,
                                              edge_port = edge_port,
                                              priority = priority,
                                              cost = cost)

@then(u'rstp status should be "{status}" on "{device}"')
def step_impl(context, status, device):
    flag = False
    for _ in range(10):
        ret = context.dut[device].main().show_rstp()
        if ret[0]["role"] == status:
            flag = True
            break
        time.sleep(1)
    assert flag, f"rstp status is not: {status}"

@then(u'"{host}" shall receive {expect_pkt_num} hello packet in {wait_time} sec')
def step_impl(context, host, expect_pkt_num, wait_time):
    context.hosts[host].tshark.start(interface=context.host_info[host]["nic"], capture_filter='stp')
    time.sleep(int(wait_time))
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve()
    pkt_num = ret.count("STP")
    assert int(pkt_num) >= int(expect_pkt_num), f"expect received packet number: {expect_pkt_num}, but {pkt_num}\nRaw: {ret}"

@then(u'"{host}" shall receive rstp packet with the content of the table')
def step_impl(context, host):
    expect_info = {"brid": "Bridge Priority",
                   "hello_time": "Hello Time",
                   "fwd_delay": "Forward Delay",
                   "max_age": "Max Age"}[context.table.headings[0]]
    context.hosts[host].tshark.start(interface=context.host_info[host]["nic"], capture_filter='stp', additional_options=f"-V | grep '{expect_info}'")
    time.sleep(5)
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve()
    expect_value = context.table[0][context.table.headings[0]]
    actual_value = re.findall(rf"{expect_info}: (\d+)", ret)

    assert all(element == expect_value for element in actual_value), \
        f"expect {expect_info}: {expect_value}, but {actual_value}"



from host import *
from behave import *
from steps import common, interface, login


@when('set password mininum length as {length} on "{device}"')
def step_impl(context, length, device):
    context.dut[device].go_config().set_password_policy(min_len=length)


@when('{action} password "{policy}" policy on "{device}"')
def step_impl(context, action, policy, device):
    global_complexity_check = item = None
    if policy == "complexity strength check":
        global_complexity_check = True
    elif policy == "must contain at least one digit":
        item = "digit"
    elif policy == "must include both upper and lower case letter":
        item = "alphabet"
    elif policy == "must contain at least one special character":
        item = "special-characters"

    context.dut[device].go_config().set_password_policy(
        action=action,
        global_complexity_check=global_complexity_check,
        complexity_item=item)



from mdc.router.cli.rp_base.lg_router.tn5000 import config
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigNat(config.Config):
    """
    Class for configuring NAT service.

        Args:
            session (object): The session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        self._s.command_expect(f'')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    # 1-1
    def set_one2one_rule(self, ipaddr_in, if_out, ipaddr_out, vrrp=None):
        cmd = f"ip nat static inside {ipaddr_in} outside {if_out} {ipaddr_out}"
        if vrrp:
            cmd += " redundancy {vrrp}"
        self._s.command_expect(cmd)

    # pat
    def set_port_forward_rule(self, protocol, ipaddr_in, port_in, if_out,
                                port_out):
        cmd = "ip nat static "
        if protocol in ["tcp", "udp", "all"]:
            cmd += f"{protocol} inside {ipaddr_in} {port_in} outside {if_out} {port_out}"
        else:
            raise ValueError(f"Input invalid protocol {protocol}!")
        self._s.command_expect(cmd)

    # n-1
    def set_dynamic_rule(self, ipaddr_start, ipaddr_end, if_out):
        cmd = f"ip nat dynamic inside {ipaddr_start} {ipaddr_end} outside {if_out}"
        self._s.command_expect(cmd)





def cli_error_detector(output_data: str):
    """
    Detects errors in console output.

        Args:
            output_data (str): The console output to check for errors.

        Raises:
            ValueError: If an error or message is detected in the console output.  Messages that match the rules in msg_ignoring_rules will be ignored. 
    """
    msg_ignoring_rules_of_hat = [lambda m: "]'." in m]
    msg_ignoring_rules_of_percentage = [lambda m: 'create vlan id:' in m,
                                        lambda m: 'Config file import failed' in m]
    # msg = output_data.split('\n', 1)[0]
    msg = output_data

    # print(f"======output_data '{output_data}' output_data end ======")
    # print(f"======msg '{msg}' msg end ======")
    head = msg.find('^')
    # print(f"======head of ^{head} cli_error_detector ======")
    if head != -1:
        tmp = output_data[head:]
        msg = tmp[1:tmp.find('\r')]
        for rule in msg_ignoring_rules_of_hat:
            if rule(msg):
                break
        else:
            # print(f"output_data: {output_data}\n")
            raise ValueError(f'error "{msg}" detected in console output')
    
    head = msg.find('% ')
    # print(f"======head of %{head} cli_error_detector ======")
    if head != -1:
        tmp = output_data[head:]
        msg = tmp[2:tmp.find('\r')]
        for rule in msg_ignoring_rules_of_percentage:
            if rule(msg):
                break
        else:
            raise ValueError(f'message "{msg}" detected in console output')


import re
import sys
import time
import random

from host import *
from behave import *
from steps import common, interface, login


@when(u'set "{device}" a static multicast table with {mac} and "{port_name}"')
def step_impl(context, device, mac, port_name):
    context.mac = mac.replace('-', ':')
    port = context.dut_info[device]['testbed'][port_name]
    vlan = context.table[0]['Vlan'] if context.table else None
    context.dut[device].go_config().set_static_multicast_table(
        mac=context.mac, port=port, vlan=vlan)
    time.sleep(10)


@then(u'the following rule should be on "{device}"\'s static multicast table')
def step_impl(context, device):
    mac_table = context.dut[device].main().show_mac_address_table()
    port = context.dut_info[device]['testbed'][context.table[0]['Port']]
    flag = False

    for entry in mac_table:
        if entry['type'] == context.table[0]['Type'] and \
            entry['mac'] == context.table[0]['MAC'].upper() and \
            entry['port'] == f'1/{port}':
            flag = True
            break
    assert flag, f'expect: [mac: {context.table[0]["MAC"].upper()},type: {context.table[0]["Type"]}, port: 1/{port}] , actual: {mac_table}'

@when(
    u'send {pkt_count} layer 2 multicast packet from "{host_send}" to {mac}'
)
def step_impl(context, pkt_count, host_send, mac):
    # fill multicast scapy script with args
    dst_mac = mac.replace('-', ':')
    src_mac = context.host_info[host_send]['testbed']['mac_address']
    dst_nic = context.host_info[host_send]["nic"]
    script = context.hosts[host_send].scapy.generate_multicast_scapy_script(
        dst_mac, src_mac, dst_nic, pkt_count)

    from datetime import datetime
    import os

    # write the scapy script to file
    scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
    with open(f'../lib/atb/service/scapy_template/{scapy_file_name}',
              "w") as script_file:
        script_content = script_file.write(script)

    # scp the file to remote host with scapy service
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(
        context.host_info[host_send]['session']['credential']['password'],
        f"../lib/atb/service/scapy_template/{scapy_file_name}",
        context.host_info[host_send]['session']['credential']['username'],
        context.host_info[host_send]['session']['host']))

    context.hosts[host_send].scapy.do_script(scapy_file_name)
    # Clean up
    os.system(f'rm ../lib/atb/service/scapy_template/{scapy_file_name}')


@then(
    u'"{host_recv}" shall "{result}" to receive layer 2 multicast packets from "{host_send}"'
)
def step_impl(context, host_recv, result, host_send):
    time.sleep(5)
    context.hosts[host_recv].tshark.stop()
    ret = context.hosts[host_recv].tshark.retrieve()

    if len(ret) == 0:
        raise ValueError('packet receiving fails')

    pkt_count = 0
    ret_lines = ret.split('\n')
    for a in ret_lines:
        arr = re.split(r'\s+', a.lstrip())
        # skip if not tshark packet
        if arr[0].isdigit() == False or len(arr) < 5:
            continue
        if arr[3] != '→':
            raise ValueError(f'unexpected tshark format {a}')

        # Match Source and Destination IP Address
        if arr[2] == context.host_info[host_send]['testbed'][
                'mac_address'] and arr[4] == context.mac:
            pkt_count += 1

    if result == 'fail':
        assert pkt_count == 0, f'{host_recv} expect to receive no packets but receive {pkt_count}'
    elif result == 'success':
        assert pkt_count > 0, f'{host_recv} expect to receive packets but receive {pkt_count}'



from scapy.all import *
from scapy.contrib.igmp import IGMP
ip_packet = IP(src="{sip}", dst="{dip}")
raw_data = bytes.fromhex("00" * {padding_n})
packet = Ether() / ip_packet / {igmp_type} / Padding(load=raw_data)
packet[IGMP].gaddr = "{igmp_group_addr}"
packet = packet.__class__(bytes(packet))
sendp(packet, iface="{iface}", count={count})


from mdc.router.cli.rp_base.lg_router.tn5000 import config
from mdc.router.cli.rp_base.lg_router.tn5000.cli import PROMPT_CONFIG

class ConfigIfWan(config.Config):
    """
    Class for configuring WAN interface.

        Args:
            session (Session): Session object to communicate with the device.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (tuple): Tuple of positional arguments.
            _kwargs (dict): Dictionary of keyword arguments. 
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__(session)
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        self._s.command_expect(f'interface wan')

    def getin(self):
        self._set_default_prompts()
        self._getin()
        return self

    def set_wan_interface(self,
                            ip,
                            mask,
                            bind_vlan: str = None,
                            mac_address: str = None,
                            gateway: str = None):
        if gateway:
            self._s.command_expect(
                f'ip address static {ip} {mask} {gateway}')
        else:
            self._s.command_expect(f'ip address static {ip} {mask}')
        if bind_vlan:
            self._s.command_expect(f'bind vlan {bind_vlan}')
        if mac_address:
            self._s.command_expect(f'mac-address {mac_address}')
        return self

    def disable_wan_interface(self):
        self._s.command_expect(f'shutdown')
        return self

    def ip_address_static(self, ip, mask, gateway=''):
        self._s.command_expect(f'ip address static {ip} {mask} {gateway}')
        return self
        
    def ip_address_dhcp(self, bind_vlan):
        """
        Set a WAN interface with dhcp mode.

            Returns: 
                self: The object instance itself.  
        """
        self._s.command_expect(f'bind vlan {bind_vlan}')
        self._s.command_expect(f'ip address dhcp')
        return self



import time

from host import *
from behave import *
from steps import common, interface, login


@then('show vlan including vlan id of "{intf}" on "{device}"')
def step_impl(context, intf, device):
    time.sleep(1)
    r = context.dut[device].main().show_vlan()
    print("-------------------------------------------")
    print(r)
    print("-------------------------------------------")
    if context.dut_info[device]['testbed'][f'{intf.lower()}_vlan'] == r[1][
            'vid']:
        print(
            f'VLAN {context.dut_info[device]["testbed"][f"{intf.lower()}_vlan"]} exist'
        )
        result = 'pass'
    else:
        result = 'fail'
    assert result == 'pass', f'VLAN {context.dut_info[device]["testbed"][f"{intf.lower()}_vlan"]} not exist'



from datetime import datetime
from session import SshSession
import os

class ScapyService:
    """
    Class for controlling a scapy container on a remote host.

        Args:
            ssh_session (SshSession): An active SSH session to the remote host.
            sudo_paswd (str): The sudo password for the remote host.

        Attributes:
            _ssh (SshSession): SSH session object.
            image_name (str): The name of the scapy image to use. Defaults to 'calico/scapy'.
            image_tag (str): The tag of the scapy image to use. Defaults to 'v2.4.0'.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self.image_name = "calico/scapy"
        self.image_tag = "v2.4.0"
        self.scapy_dict_name = ""
        self.scapy_file_name = ""

    def _check_conneted(self):
        """
        Check if the SSH session is connected.

            Raises:
                ValueError: If the SSH session is not connected.
        """
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')

    def check_env(self):
        """
        Checks if the environment is set up correctly.

            Raises:
                ValueError: If the SSH session is not connected or if the docker image is not found.
        """
        self._check_conneted()
        output = self._ssh.command('docker images')
        if self.image_name not in output:
            raise ValueError(f'No docker image {self.image_name}')

    def send(self, scapy_syntax: str = '', dev: str = None, count: int = 1):
        """
        Send packet with Scapy.

            Args:
                scapy_syntax (str, optional): Scapy syntax for packet format. Defaults to ''.
                dev (str, optional): Device name. Defaults to None.
                count (int, optional): Number of packets to send. Defaults to 1.

            Returns:
                str: Output of the command execution. 
        """
        self._check_conneted()

        scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
        scapy_dict_name = "scapy_script"

        self._ssh.command(f'mkdir -p $HOME/{scapy_dict_name}')
        if dev is not None:
            scapy_call = f'sendp({scapy_syntax}, iface="{dev}", count={count})'
        else:
            scapy_call = f'send({scapy_syntax}, count={count})'
        scapy_file_out_cmd = f'printf \'{scapy_call}\' >> $HOME/{scapy_dict_name}/{scapy_file_name}'
        self._ssh.command(scapy_file_out_cmd)

        # Descript packet format in `scapy_dict_name` and send packet with Scapy.
        docker_cmd = f'docker run --rm --net=host \
            -v $HOME/{scapy_dict_name}:/{scapy_dict_name} {self.image_name}:{self.image_tag} \
            scapy -c /{scapy_dict_name}/{scapy_file_name}'

        ret = self._ssh.command(docker_cmd)

        # Clean up
        print(docker_cmd)
        self._ssh.command(f"rm $HOME/{scapy_dict_name}/{scapy_file_name}")
    

    def generate_igmp_scapy_script(self, 
                                   sip: str, 
                                   dip: str, 
                                   padding_n: str, 
                                   igmp_type: str, 
                                   igmp_group_addr: str, 
                                   count: str, 
                                   iface: str):
        """
        Fill the igmp scapy script template with the given parameters
        Args:
            sip (str): Source IP
            dip (str): Destination IP
            padding_n (str): Padding number
            igmp_type (str): IGMP type
            igmp_group_addr (str): IGMP group address
            count (str): Number of packets to send
            iface (str): Interface name

        Returns:
            str: filled_script: The filled script with the given parameters.
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/igmp_script.py', "r") as script_file:
            script_content = script_file.read()

        filled_script = script_content.format(
            sip=sip,
            dip=dip,
            padding_n=padding_n,
            igmp_type=igmp_type,
            igmp_group_addr=igmp_group_addr,
            iface=iface,
            count=count
        )
        # print("filled_script",filled_script)
        return filled_script


    def generate_arp_scapy_script(self, 
                                  source_macs: list, 
                                  ips: list, 
                                  target_ip: str, 
                                  nic: str):
        """
        Fill the arp scapy script template with the given parameters
        Args:
            source_macs (list): list of source macs
            ips (list): list of ips
            target_ip (str): target ip
            nic (str): nic name

        Returns:
            str: filled_script: The filled script with the given parameters.
            
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/arp_script.py', "r") as script_file:
            script_content = script_file.read()

        filled_script = script_content.format(
            source_macs=source_macs,
            ips=ips,
            target_ip=target_ip,
            nic=nic
        )
        # print("filled_script",filled_script)
        return filled_script

    def generate_vrrp_scapy_script(self, 
                                   padding_n: int, 
                                   smac: str, 
                                   dmac: str, 
                                   sip: str, 
                                   dip: str, 
                                   version: int, 
                                   priority: int, 
                                   ip_address: str, 
                                   iface: str):
        """
        Fill the broadcast scapy script template with the given parameters
        Args:
            padding_n (int): Padding number
            smac (str): Source MAC
            dmac (str): Destination MAC
            sip (str): Source IP
            dip (str): Destination IP
            version (int): Version
            priority (int): Priority
            ip_address (str): IP address
            iface (str): Interface

        Returns:
            str: filled_script: The filled script with the given parameters.
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/vrrp_script.py', "r") as script_file:
            script_content = script_file.read()
        filled_script = script_content.format(
            padding_n=padding_n,
            smac=smac,
            dmac=dmac,
            sip=sip,
            dip=dip,
            version=version,
            priority=priority,
            ip_address=ip_address,
            iface=iface
        )
        # print("filled_script",filled_script)
        return filled_script

    def generate_broadcast_scapy_script(self, 
                                      sip: str, 
                                      dip: str, 
                                      dport: int, 
                                      iface: str,
                                      count: str):
        """
        Fill the broadcast scapy script template with the given parameters
        Args:
            sip (str): Source IP
            dip (str): Destination IP
            dport (int): Destination port
            iface (str): Interface
            count (str): Number of packets to send

        Returns:
            str: filled_script: The filled script with the given parameters.
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/broadcast_script.py', "r") as script_file:
            script_content = script_file.read()

        filled_script = script_content.format(
            sip=sip,
            dip=dip,
            dport=dport,
            iface=iface,
            count=count
        )
        # print("filled_script",filled_script)
        return filled_script
    
    def generate_rstp_scapy_script(self, 
                                   smac: int, 
                                   dmac: str, 
                                   brid: str, 
                                   iface: int,
                                   max_age: int,
                                   path_cost: int, 
                                   hello_time: int,
                                   fwd_delay: int,
                                   padding_n: int):
        """
        fill the vrrp scapy script template with the given parameters
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/rstp_script.py', "r") as script_file:
            script_content = script_file.read()
        filled_script = script_content.format(smac=smac,
                                              dmac=dmac,
                                              brid=brid,
                                              iface=iface,
                                              max_age=max_age,
                                              path_cost=path_cost, 
                                              hello_time=hello_time,
                                              fwd_delay=fwd_delay,
                                              padding_n=padding_n)
        # print("filled_script",filled_script)
        return filled_script
    
    def generate_rip_scapy_script(self, 
                                  rip_type: str,
                                  smac: int, 
                                  dmac: int, 
                                  sip: str, 
                                  dip: str, 
                                  iface: str,
                                  ripentries: list = None):
        
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/rip_script.py', "r") as script_file:
            script_content = script_file.read()

        filled_script = script_content.format(
            rip_type=rip_type,
            smac=smac, 
            dmac=dmac, 
            sip=sip, 
            dip=dip, 
            iface=iface,
            ripentries=ripentries
        )
        # print("filled_script",filled_script)
        return filled_script

    def generate_multicast_scapy_script(self, 
                                        dst_mac: str, 
                                        src_mac: str, 
                                        nic: str, 
                                        pkt_count: str):
        """
        Fill the multicast scapy script template with the given parameters.
        Args:
            dst_mac (str): destination MAC address
            src_mac (str): source MAC address
            nic (str): nic name
            pkt_count (str): number of packets to send
        
        Return:
            str: filled_script: The filled script with the given parameters.
        """
        with open(os.path.dirname(os.getcwd())+ '/lib/atb/service/scapy_template/multicast_script.py', "r") as script_file:
            script_content = script_file.read()

        filled_script = script_content.format(dst_mac=dst_mac, src_mac=src_mac, nic=nic, pkt_count=int(pkt_count))

        return filled_script

    def do_script(self, scapy_file_name: str = ''):
        self._check_conneted()

        self.scapy_dict_name = "scapy_script"
        self.scapy_file_name = scapy_file_name

        self._ssh.command(f'mkdir -p $HOME/{self.scapy_dict_name}')
        self._ssh.command(f'mv $HOME/{self.scapy_file_name} $HOME/{self.scapy_dict_name}/')
        docker_cmd = f'docker run --rm --net=host \
            -v $HOME/{self.scapy_dict_name}:/{self.scapy_dict_name} {self.image_name}:{self.image_tag} \
            scapy -c /{self.scapy_dict_name}/{self.scapy_file_name}'

        ret = self._ssh.command(docker_cmd)

    def clean_up(self):
        # Clean up
        self._ssh.command(f"rm $HOME/{self.scapy_dict_name}/{self.scapy_file_name}")




from host import *
from behave import *
from steps import common, interface, login


@given('clear ssh and telnet limit on "{device}"')
def step_impl(context, device):
    '''
    This step may require modification for future usage.
    Due to the DUT's limit on the number of SSH and Telnet sessions, 
    we need to clear the limitation before logging in and out multiple times.
    '''
    context.dut[device].main().clear_ssh_telnet_limit()
    context.execute_steps(f'''given logout "{device}"''')
    context.execute_steps(f'''given login "{device}" as Admin''')


@then('no {user} on "{device}" User Accounts')
def step_impl(context, device, user):
    ret = context.dut[device].main().show_user_accounts()
    for i in ret:
        if user == i['Name']:
            assert False, f'{user} should not be on User Accounts list'



from host import *
from behave import *
from steps import common, interface, login


@when(u'test "{device}"')
def step_impl(context, device):
    pass
    # context.hosts['HOST_EXECUTOR'].shellcmd.show_network()
    # context.hosts['HOST_A'].shellcmd.show_network()
    # context.hosts['HOST_B'].shellcmd.show_network()
    # context.hosts['HOST_B'].shellcmd.unzip_file('qq', 'qq', 'qq')
    # context.hosts['HOST_B'].shellcmd.arp(4, '192.168.127.95')
    # context.hosts['HOST_B'].tcpdump.tcpdump('eth1', 'qq')
    # context.hosts['HOST_B'].tcpdump.get_tcpdumpfile('123')
    # pass
    # context.dut[device].main().set_backdoor(
    #     context.dut_info[device]['credential']['root_account'],
    #     context.dut_info[device]['credential']['root_password'])
    # device = 'DUT'
    # context.execute_steps(f'''given authorize CLI of "{device}"''')
    # context.dut[device].main().show_system()
    # info = context.host_info['HOST_EXECUTOR']
    # context.hosts['HOST_EXECUTOR'] = make_host(info)
    # context.hosts['HOST_EXECUTOR'].login()
    # context.hosts['HOST_EXECUTOR'].tftp.check_env()
    # print(context.dut["DUT"].main().show_dos())
    # print(context.dut["DUT"].main().show_system())
    # print(context.dut["DUT"].main().show_version())
    # print(context.dut["DUT"].main().show_l37_policy())
    # print(context.dut["DUT"].main().show_logging_event_log_dos())
    # print(context.dut["DUT"].main().show_logging_event_log_system())
    # print(context.dut["DUT"].main().show_logging_event_log_l3l7())
    # print(context.dut["DUT"].main().show_interface('lan'))
    # print(context.dut["DUT"].main().show_interface('wan'))
    # print(context.dut["DUT"].main().show_interface('bridge'))
    # print(context.dut["DUT"].main().show_trusted_access())
    # context.dut["DUT"].go_config_ipsec('T1').go_phase1().command('qq')
    # context.dut["DUT"].go_config_ipsec('T1').go_phase2().command('qq')
    # info = context.host_info['HOST']
    # context.host = make_host(info)
    # context.host.login()
    # context.dut["DUT"].go_config().enable_service_dhcp()
    # context.dut["DUT"].go_config().enable_service_dhcp('auto-assign')
    # context.dut["DUT"].go_config_dhcp(1). \
    # network('192.168.127.1', '192.168.127.254', '255.255.255.0'). \
    # lease('1440'). \
    # dns_server('8.8.8.8'). \
    # ntp_server('192.168.127.32'). \
    # default_router('192.168.127.254'). \
    # exit()
    # context.dut["DUT"].go_config_dhcp('qq'). \
    # host('192.168.127.93', '255.255.255.0'). \
    # hardware_address('00:0c:29:49:cd:1e'). \
    # lease('1440'). \
    # dns_server('8.8.8.8'). \
    # ntp_server('192.168.127.32'). \
    # default_router('192.168.127.254'). \
    # exit()

    # context.host.shellcmd.enable_dhcp_client(dev='ens224')



import time
from mdc.router.cli import common_func
from mdc.router.cli.rp_parser import parser
from mdc.router.cli.rp_base.lg_router.edr800 import cli
from mdc.router.cli.rp_base.lg_router.edr800.cli import PROMPT_MAIN

class Main(cli.Cli):
    """
    Class for operating command at main prompt

        Args:
            session (obj): Session object.
            model (str): Model of the device.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _model (str): Model of the device.
    """

    def __init__(self, session, model):
        super().__init__(session)
        self._model = model
        self._set_default_prompts()
        self._back_to_main()

    def _set_default_prompts(self):
        self._s.set_default_prompts([PROMPT_MAIN])

    def exit(self):
        self._s.command_expect('exit')

    def ping(self, host):
        return parser.ping(self._s, host, self._model)

    def show_version(self):
        return parser.show_version(self._s, self._model)

    def show_system(self):
        return parser.show_system(self._s, self._model)

    def show_interface(self, name='lan'):
        return parser.show_interface(self._s, name, self._model)

    def show_vlan(self):
        return parser.show_vlan(self._s, self._model)

    def show_clock(self):
        return parser.show_clock(self._s, self._model)

    def reload_factory_default(self):
        common_func.reload_factory_default(self._s)
    
    def reload_factory_default_no_cert(self):
        common_func.reload_factory_default_no_cert(self._s)

    def show_dos(self):
        return parser.show_dos(self._s, self._model)

    def show_ipsec(self, connection_name=None):
        if connection_name is None:
            ret = self._s.command_expect('show ipsec')
        else:
            ret = self._s.command_expect(f'show ipsec {connection_name}')
        return ret['data']

    def show_ipsec_status(self):
        ret = self._s.command_expect('show ipsec status')
        return ret['data']

    def show_ip_route(self):
        return parser.show_ip_route(self._s, self._model)

    # def show_logging_event_log_dos(self):
    #     return parser.show_logging_event_log_dos(self._s, self._model)

    def show_ip_igmp(self):
        return parser.show_ip_igmp(self._s, self._model)

    def show_logging_event_log_firewall(self):
        return parser.show_logging_event_log_firewall(
            self._s, self._model)

    def show_logging_event_log_system(self):
        return parser.show_logging_event_log_system(
            self._s, self._model)

    def show_user_accounts(self):
        return parser.show_user_accounts(self._s, self._model)

    def show_syslog_setting(self):
        self.command('show logging')

    # no parsing required for now
    def show_arp(self):
        self.command('show arp')

    def show_trusted_access(self):
        return parser.show_trusted_access(self._s, self._model)

    def reload(self):
        common_func.reload(self._s)

    def login(self):
        common_func.login(self._s)

    def clear_event_log(self):
        self.set_backdoor()
        self._s.command_expect('cd /mnt/log2', prompts=[r'/mnt/log2 #'])
        self._s.command_expect('cd /mnt/firewall_LOG',
                                prompts=[r'/mnt/firewall_LOG #'])
        self._s.command_expect('rm EDR900_LOG1', prompts=[r'/mnt/log2 #'])
        self._s.command_expect('rm firewall_log', prompts=[r'/mnt/log2 #'])
        self._s.command_expect('exit')
        # disable ctrl-t mode for serial control
        self._s.sendcontrol('t')

    def set_backdoor(self):
        self._s.sendcontrol('t')
        cmd_args = [
            ('ieisecureedr moxaiei89191230', [r'~ #']),
        ]
        for args in cmd_args:
            r = self._s.command_expect(args[0], prompts=args[1])
            if not r['matched']:
                print(f"return = {r}, command = {args[0]}")
                raise Exception("failed to setup backdoor")
        return self

    def clear_ssh_telnet_limit(self):
        '''
        This step may require modification for future usage.
        Due to the DUT's limit on the number of SSH and Telnet sessions, 
        we need to clear the limitation before logging in and out multiple times.
        '''
        self.set_backdoor()
        self._s.command_expect('iptables -F PortScan')
        self._s.command_expect('exit')
        # disable ctrl-t mode for serial control
        self._s.sendcontrol('t')

    def import_config(self,
                        tftp_ip: str = None,
                        usb: bool = False,
                        file_name: str = "test.ini"):
        if tftp_ip != None and usb == False:
            if file_name == 'configuration file':
                return self._s.command_expect(
                    f'copy tftp {tftp_ip} config-file test.ini',
                    prompts=[PROMPT_MAIN],
                    timeout=60)
            else:
                return self._s.command_expect(
                    f'copy tftp {tftp_ip} config-file {file_name}',
                    prompts=[PROMPT_MAIN],
                    timeout=60)
        elif tftp_ip == None and usb == True:
            if file_name == 'configuration file':
                return self._s.command_expect(
                    f'copy usb test.ini',
                    prompts=[PROMPT_MAIN],
                    timeout=60)
            else:
                return self._s.command_expect(
                    f'copy usb {file_name}',
                    prompts=[PROMPT_MAIN],
                    timeout=60)

    def export_config(self, tftp_ip, file_name="test.ini"):
        return self._s.command_expect(
            f'copy running-config tftp {tftp_ip} {file_name}',
            prompts=[PROMPT_MAIN], timeout=60)

    def get_running_config(self, con_type='serial'):
        running_config = ""
        ret = self._s.command_expect('show running-config',
                                        prompts=[PROMPT_MAIN],
                                        timeout=5)
        running_config += ret['data']
        while ret['matched'] == False:
            ret = self._s.command_expect('',
                                            prompts=[PROMPT_MAIN],
                                            timeout=1)
            running_config += ret['data']
        if con_type == 'telnet':
            return ret['data'].replace('\n--More--', '\n')
        else:
            return running_config.replace('\n--More--', '\n')

    def upgrade_firmware(self,
                            tftp_ip,
                            file_name):
        ret = self._s.command_expect(
            f'copy tftp {tftp_ip} device-firmware {file_name}',
            prompts=['System will reboot now.'],
            timeout=300)
        time.sleep(60)
        return ret

    def save(self):
        return self._s.command_expect('save',
                                        prompts=[PROMPT_MAIN],
                                        exact_str=True,
                                        timeout=30)
    def show_rstp(self):
        """
         Show rstp global information in human readable form. This is useful for debugging purposes.
         
         
            Returns: 
                A string of RLP formatted to be displayed on screen
        """
        return parser.show_rstp(self._s, self._model)

    def show_vrrp(self):
        """
         Show VRRP in human readable form. This is useful for debugging purposes.
         
         
            Returns: 
                A string of RLP formatted to be displayed on screen
        """
        return parser.show_vrrp(self._s, self._model)

    def show_mac_address_table(self):
        """
        Execute the 'show mac-address-table' command on the device.

            Returns:
                str: Output of 'show mac-address-table' command.
        """
        return parser.show_mac_address_table(self._s, self._model)

    def show_fast_bootup(self):
        return parser.show_fast_bootup(self._s, self._model)



from mdc.router.cli.md_tn5916.tn5916_v3 import tn5916_v3_0

class tn5916_v3_0_27(tn5916_v3_0):
    class ConfigIfEthernet(tn5916_v3_0.ConfigIfEthernet):
        def set_bridge_group(self, group: int = None):
                self._s.command_expect(f'bridge group')
                return self



from utils.input_chk import chk_valid_ip
from mdc.router.cli.rp_base.ng_router import cli
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class Config(cli.Cli):
    """
    Class for configuring a device through the command line interface.

        Args:
            session (Session): A Session object used to communicate with the device.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
    """

    def __init__(self, session: object):
        super().__init__(session)
        # back to main
        self._back_to_main()
        # jumpt to config terminal
        self.getin_cfg()

    def get_session(self):
        """
        Return the Session object used to communicate with the device.

            Returns: 
                Session: The Session object used to communicate with the device.
        """
        return self._s

    def _set_default_prompts_cfg(self):
        """
        Set default prompts for configuration mode.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin_cfg(self):
        """
        Enter configuration mode.
        """
        self._s.command_expect('configure terminal')

    # function override is allowed
    def getin_cfg(self):
        """
        Enter configuration mode and set default prompts for configuration mode. 

            Returns: 
                Session: The Session object used to communicate with the device in configuration mode.
        """
        self._set_default_prompts_cfg()
        self._getin_cfg()
        return self

    class MailServer:
        """
        A class representing a mail server.

            Attributes:
                _s (object): The session object used to interact with the mail server.

            Methods:
                set_server(server: str, port: int): Sets the server and port for the mail server.
                set_mail_address(mail_address: str): Sets the mail address for the mail server.
                set_account(account: str, paswd: str): Sets the account and password for the mail server.
                set_sender(mail_address: str): Sets the sender mail address for the mail server.
                send_test(self): Sends a test email from the mail server.
        """
        def __init__(self, session: object):
            self._s = session
        def set_server(self, server, port):
            """
            Sets the server and port for the email-warning command.

                Args:
                    server (str): The server to set.
                    port (int): The port to set.
            """
            self._s.command_expect(f'email-warning server {server} {port}')
        def set_mail_address(self, mail_address):
            """
            Sets the email address for receiving warnings.

                Args:
                    mail_address (str): The email address to set.
            """
            self._s.command_expect(f'email-warning mail-address 1 {mail_address}')
        def set_account(self, account, paswd):
            """
            Sets the account and password for the router.

                Args:
                    account (str): The account to set.
                    paswd (str): The password to set.
            """
            self._s.command_expect(f'email-warning account {account} {paswd}')
        def set_sender(self, mail_address):
            """
            Sets the sender email address for email warnings.

                Args:
                    mail_address (str): The email address to set as the sender.
            """
            self._s.command_expect(f'email-warning sender {mail_address}')
        def send_test(self):
            """
            Sends a test email warning using the `_s` instance.
            """
            self._s.command_expect(f'email-warning send')

    # ======== dhcp feature ========
    class DhcpService:
        """Class to configure DHCP service.

            Args:
                session (object): An instance of the Session class.
        """

        def __init__(self, session: object):
            self._s = session

        def set_global_dhcp_service(self,
                                    action: str = "enable",
                                    mode: str = ""):
            """
            Configure global DHCP service.
            
                Args
                    mode: global dhcp mode ("" or auto-assign).
                    action: global dhcp action (enable or disable).
            """
            cmd = "service dhcp"
            if action == "enable":
                cmd = f"{cmd} {mode}"
            elif action == "disable":
                cmd = f"no {cmd} {mode}"
            else:
                raise ValueError(f"input invalid action: {action}")

            self._s.command_expect(cmd)

        def set_global_dhcp_server(self,
                                    mode: str = "pool",
                                    action: str = "enable",
                                    index: str = "1",
                                    name: str = "dhcp_server"):
            """
            Configure global DHCP server.

            Args
                mode: dhcp server mode (pool or static pool).
                name: dhcp server name. Defaults to "dhcp_server".
                index: dhcp server index. Defaults to "1".
                action: dhcp server action (enable or disable).
            """
            cmd = "ip dhcp"
            if mode == "pool":
                cmd = f"{cmd} {mode} {index} {action}"
            elif mode == "static pool":
                cmd = f"{cmd} {mode} {name} {action}"
            else:
                raise ValueError(f"input invalid dhcp mode: {mode}")

            self._s.command_expect(cmd)

    # ======== End of dhcp feature ========
    # ======== login lockout feature ========
    class LoginLockout:
        """
        Class to configure login lockout settings on a device.

            Args:
                session (Session): Instance of Session class to communicate with the device.

            Attributes:
                _s (Session): Instance of Session class to communicate with the device.
                _cmd (str): Command used to configure login lockout settings.
        """

        def __init__(self, session: object):
            self._s = session
            self._cmd = 'login-lockout'

        def lockout_toggle(self, toggle: bool):
            """
            Enable or disable login lockout on the device.

                Args:
                    toggle (bool): True to enable, False to disable. 
            """
            if toggle is True:
                self._s.command_expect(f'{self._cmd}')
            else:
                self._s.command_expect(f'no {self._cmd}')

        def lockout_retryThreshold(self, retry: str):
            """
            Configure the number of failed login attempts before locking out an account.

                Args: 
                    retry (str): Number of failed login attempts before locking out an account. 
            """
            self._s.command_expect(f'{self._cmd} retry-threshold {retry}')

        def lockout_lockoutTime(self, duration: str):
            """
            Configure the duration for which an account is locked out after exceeding the maximum number of failed login attempts. 

                Args: 
                    duration (str): Duration in minutes for which an account is locked out after exceeding the maximum number of failed login attempts.  
            """
            self._s.command_expect(f'{self._cmd} lockout-time {duration}')

    # ======== End of login lockout feature ========
    # ======== time sync feature ========
    class TimeSync:
        """
        Class for synchronizing time with a remote server or manually setting the time.

            Args:
                session (object): Session object to send commands to the device.
        """

        def __init__(self, session: object):
            self._s = session

        def clock_source(self, clk_src: str, remote_server: str = None):
            """
            Set the clock source for the device.

                Args:
                    clk_src (str): Clock source to set, either 'local', 'ntp' or 'sntp'.
                    remote_server (str): IP address of the remote server to sync with. Required if clk_src is 'ntp' or 'sntp'.

                Raises:
                    Exception: If an unsupported clock source is provided. 
            """
            if clk_src.lower() == "local":
                self._s.command_expect(f'no ntp remote-server')
            elif clk_src.lower() == "ntp" or clk_src.lower() == "sntp":
                chk_valid_ip(remote_server)
                cmd = f'ntp remote-server {remote_server}'
                if clk_src.lower() == "sntp":
                    cmd += ' simple'
                self._s.command_expect(cmd)
            else:
                raise Exception(f"Unsupported Clock Source: {clk_src}")

        def ntp_server_toggle(self, toggle):
            """
            Toggle the NTP server on or off.

                Args:
                    toggle (bool): True to turn on the NTP server, False to turn it off.
            """
            if toggle is True:
                self._s.command_expect(f'ntp server')
            else:
                self._s.command_expect(f'no ntp server')

        def set_time_manually(self, year: str, month: str, day: str,
                                hour: str, min: str, sec: str):
            """
            Set the time manually for the device.

                Args: 
                    year (str): The year to set the time to. 
                    month (str): The month to set the time to. 
                    day (str): The day to set the time to. 
                    hour (str): The hour to set the time to. 
                    min (str): The minute to set the time to. 
                    sec (str): The second to set the time to. 
            """
            self._s.command_expect(
                f'clock set {hour}:{min}:{sec} {month} {day} {year}')

    # ======== End of time sync feature ========

    def set_clock_summer_time(self,
                                start_date: str = None,
                                end_date: str = None,
                                offset: str = None):
        """
        Set the Daylight Saving for the device.
            Args: 
                start_date (str): The Date when Summer Time Offset Start. The format should be "{month} {week} {day} {hour} {min}" 
                end_date (str): The Date when Summer Time Offset End. The format should be "{month} {week} {day} {hour} {min}"
                offset (str): Summer Time Offset. (1 - 12)
            Explanation of the Time format:
                month: From 'Jan', 'January' or '1' to 'Dec', 'December', or '12'
                week: From '1st' or '1' to 'Last' or '6'
                day: From 'Sun', 'Sunday' or '1' to 'Sat', 'Saturday' or '7'
                hour: 0 - 23
                min: 0 - 59
        """
        if start_date:
            self._s.command_expect(
                f'clock summer-time start-date {start_date}')
        if end_date:
            self._s.command_expect(f'clock summer-time end-date {end_date}')
        if offset:
            self._s.command_expect(f'clock summer-time offset {offset}')

    def set_hostname(self, hostname: str):
        """
        Sets the hostname of the device.

            Args:
                hostname (str): The name of the device. 
        """
        self._s.command_expect(command=f'hostname {hostname}')

    def set_login_account(self,
                            action: str,
                            username: str,
                            password: str,
                            privilege: str = None):
        """
        Sets the login account for the device.

            Args: 
                action (str): The action to be taken on the login account. Can be either "create", "modify" or "delete". 
                username (str): The username of the login account. 
                password (str): The password of the login account. 
                privilege (str, optional): The privilege level of the login account. Can be either "system admin", "configuration admin", "user" or "no login". Defaults to None.
        """
        cmd = ""
        if username:
            cmd += f"username {username} "
        if password:
            cmd += f"password {password} "

        if privilege != None:
            if privilege == "system admin":
                cmd += "privilege 1"
            elif privilege == "configuration admin":
                cmd += "privilege 2"
            elif privilege == "user":
                cmd += "privilege 3"
            elif privilege == "no login":
                cmd += "privilege 4"
            else:
                raise ValueError(f'input invalid privilege: {privilege}')

        if action != None:
            if action == "delete":
                cmd = f"no {cmd}"
            elif action in ["create", "modify"]:
                pass
            else:
                raise ValueError(f'input invalid action: {action}')
        return self._s.command_expect(cmd, prompts=["#"], exact_str=True)

    def set_password_policy(self,
                            min_len: str = None,
                            action: str = None,
                            global_complexity_check: bool = None,
                            complexity_item: str = None):
        """
        Set password policy.

            Args:
                min_len (str): Minimum length of the password.
                action (str): Enable or disable the password policy.
                global_complexity_check (bool): Enable or disable the global complexity check of the password.
                complexity_item (str): Complexity item of the password.

            Returns:
                str: The output of command execution. 
        """
        cmd = "password-policy "
        if min_len:
            cmd += f"minimum-length {min_len}"
        elif global_complexity_check:
            cmd += f"complexity-check"

        if complexity_item != None:
            if complexity_item == "digit":
                cmd += f"complexity-check digit"
            elif complexity_item == "alphabet":
                cmd += f"complexity-check alphabet"
            elif complexity_item == "special-characters":
                cmd += f"complexity-check special-characters"
            else:
                raise ValueError(
                    f'input invalid complexity item: {complexity_item}')

        if action != None:
            if action == "disable":  # can only disable complexity
                cmd = f"no {cmd}"
            elif action == "enable":
                pass
            else:
                raise ValueError(f'input invalid action: {action}')

        return self._s.command_expect(cmd, prompts=["#"], exact_str=True)

    def set_global_l3l7_policy(self,
                                global_action: str = None,
                                default_action: str = None,
                                index: str = None,
                                index_action: str = None):
        """
        Set global l3l7 policy.

            Args:
                global_action (str): Global action.
                default_action (str): Default action.
                index (str): Index of the policy.
                index_action (str): Action of the policy with given index. 
            Returns:
                self: The instance itself. 
        """
        cmd = "l3l7-policy "
        if global_action:
            cmd += global_action
        elif default_action:
            cmd += f"default-action {default_action}"
        elif index:
            cmd += f"{index} {index_action}"
        self._s.command_expect(cmd, prompts=[PROMPT_CONFIG])
        return self

    def set_log(self, log_item: str = None):
        """
        Set log item for logging. 
            Args: 
                log_item (str): Log item to be set for logging, valid items are "trusted-access", "dos", "ipsec", "firewall", and "l3l7-policy". 
            Returns: 
                self: The instance itself.
        """
        log_list = [
            "trusted-access", "dos", "ipsec", "firewall", "l3l7-policy"
        ]
        if log_item in log_list:
            self._s.command_expect(f"logging {log_item}",
                                    prompts=[PROMPT_CONFIG])
        else:
            raise ValueError(f'input invalid log item: {log_item}')
        return self

    def set_syslog_server(self,
                            action: str,
                            ip: str,
                            port: str = '514',
                            index: str = '1'):
        """
        Set the syslog server configuration.

            Args:
                action (str): The action to perform on the syslog server. Can be either "create", "enable" or "disable".
                ip (str): The IP address of the syslog server. 
                port (str, optional): The port number of the syslog server. Defaults to 514. 
                index (str, optional): The index of the syslog server. Defaults to 1. 
                
            Raises:
                ValueError: If an invalid action is given as input. 
        """
        if action == "create":  # create server will enable itself as well
            self._s.command_expect(f"logging {ip} {port} {index}",
                                    prompts=[PROMPT_CONFIG])
        elif action in ["enable", "disable"]:
            self._s.command_expect(f"logging {ip} {port} {action}",
                                    prompts=[PROMPT_CONFIG])
        else:
            raise ValueError(f'input invalid action: {action}')

    def set_warning_notification(self,
                                    event: str,
                                    action: str = None,
                                    severity: str = None,
                                    active: bool = None):
        """
        Set warning notification for system events.

            Args:
                event (str): System event type. Valid types are cold-start, warm-start, pwr1-trans-on, pwr2-trans-on, pwr1-trans-off, pwr2-trans-off, auth-fail, topology-changed, coupling-changed, master-changed, di1-trans-on, di1-trans-off, fiber-warning, vrrp-state changed, dot1x auth fail, firewall policy changed and config changed.
                action (str): Action type for the system event. Valid types are SNMP Trap Server only, Email only , SNMP Trap Server+Email , Syslog only , SNMP Trap Server+Syslog Server , Email+Syslog Server , SNMP Trap Server+Email+Syslog Server , Relay1 only , SNMP Trap Server+Relay1 , Email+Relay1 , SNMP Trap Server+Email+Relay1 , Syslog+Relay1 , SNMP Trap Server+Syslog Server+Relay1 , Email+Syslog Server+Relay1 and SNMP Trap Server +Email +Syslog Server +Relay1. 
                severity (str): Severity type for the system event. Valid types are Emergency 0 Alert 1 Critical 2 Error 3 Warning 4 Notice 5 Information 6 Debug 7 
                active (bool): Whether to enable the warning notification for the system event or not. 
                
            Raises:
                ValueError: If input invalid event type. 
        """
        action_type = {
            "SNMP Trap Server only": 1,
            "Email only": 2,
            "SNMP Trap Server+Email": 3,
            "Syslog only": 4,
            "SNMP Trap Server+Syslog Server": 5,
            "Email+Syslog Server": 6,
            "SNMP Trap Server+Email+Syslog Server": 7,
            "Relay1 only": 8,
            "SNMP Trap Server+Relay1": 9,
            "Email+Relay1": 10,
            "SNMP Trap Server+Email+Relay1": 11,
            "Syslog+Relay1": 12,
            "SNMP Trap Server+Syslog Server+Relay1": 13,
            "Email+Syslog Server+Relay1": 14,
            "SNMP Trap Server+Email+Syslog Server+Relay1": 15
        }
        event_type = [
            "cold-start", "warm-start", "pwr1-trans-on", "pwr2-trans-on",
            "pwr1-trans-off", "pwr2-trans-off", "auth-fail",
            "topology-changed", "coupling-changed", "master-changed",
            "di1-trans-on", "di1-trans-off", "fiber-warning",
            "vrrp-state-changed", "dot1x-auth-fail",
            "firewall-policy-changed", "config-changed"
        ]
        severity_type = {
            "Emergency": 0,
            "Alert": 1,
            "Critical": 2,
            "Error": 3,
            "Warning": 4,
            "Notice": 5,
            "Information": 6,
            "Debug": 7
        }
        if event in event_type:
            if active == True:
                self.command(
                    f"warning-notification system-event {event} active")
            if action in action_type:
                self.command(
                    f"warning-notification system-event {event} action {action_type[action]}"
                )
            if severity in severity_type:
                self.command(
                    f"warning-notification system-event {event} severity {severity_type[severity]}"
                )
        else:
            raise ValueError(f'input invalid event: {event}')

    def set_trusted_access(self,
                            ip: str = None,
                            mask: str = None,
                            action: str = None,
                            global_toggle: str = None):
        """
        Set trusted access on the device. 

            Args:
                ip (str): IP address of the trusted access.
                mask (str): Mask of the trusted access.
                action (str): Action to be taken on the trusted access. Must be either "enable" or "delete".
                global_toggle (str): Global toggle to enable/disable the trusted access.

            Raises:
                ValueError: If an invalid action is provided.
        """
        if global_toggle:
            if global_toggle == "enable":
                self.command(f"interface trusted-access")
            elif global_toggle == "disable":
                self.command(f"no interface trusted-access")
            else:
                raise ValueError(
                    f'input invalid global_toggle: {global_toggle}')
        if ip and mask:
            if action == "enable":
                self.command(f"interface trusted-access {ip} {mask} enable")
            elif action == "disable":
                self.command(
                    f"interface trusted-access {ip} {mask} disable")
            elif action == "delete":
                self.command(f"no interface trusted-access {ip} {mask}")
            else:
                raise ValueError(f'input invalid action: {action}')

    def set_vlan_id(self, vlan_id: str, delete=False):
        """
        Set VLAN ID on the device.

            Args: 
                vlan_id (str): VLAN ID to be set on the device.
                delete (bool): defualt as False, which means default as creating vlan

            Returns: 
                self: Return self for method chaining. 
        """
        if delete == False:
            self._s.command_expect(f"vlan create {vlan_id}")
        elif delete == True:
            self._s.command_expect(f"no vlan create {vlan_id}")
        return self

    def set_ip_ping_response(self):
        """
        Enable IP ping response on the device.

            Returns: 
                self: Return self for method chaining. 
        """
        self._s.command_expect("ip ping-response")
        return self

    def set_ip_http_max_login_users(self, max_users: str):
        """
        Set maximum login users for HTTP server on the device.

            Args: 
                max_users (str): Maximum login users for HTTP server to be set on the device.

            Returns: 
                self: Return self for method chaining.
        """
        self._s.command_expect(
            f"ip http-server max-login-users {max_users}")
        return self

    def set_global_telnet(self,
                            port: int = None,
                            max_login_users: int = None):
        """
        Set the global telnet configuration.

            Args:
                port (int, optional): The port to use for telnet. Defaults to None.
                max_login_users (int, optional): The maximum number of users allowed to log in via telnet. Defaults to None.
        """
        if port != None:
            self._s.command_expect(f"ip telnet port {port}")
            self._s.command_expect(f"ip telnet")
        if max_login_users != None:
            self._s.command_expect(
                f"ip telnet max-login-users {max_login_users}")

    def set_ntp_auth(self, key_id: str, key_type: str, key: str):
        """
        Create NTP Authentication Keys on the device. 

            Args:
                key_id (str): Key ID (1~65535)
                key_type (str): "MD5" or "SHA512"
                key (str): Key String (max. 32 characters)
        """
        self._s.command_expect(
            f"ntp authentication-key {key_id} {key_type} {key}")
        return self

    def set_l2tp_user(self, name: str, pwd: str):
        self._s.command_expect(f"l2tp user {name} password {pwd}")
        return self

    def set_email(self, name: str = None, pwd: str = None):
        if name and pwd:
            self._s.command_expect(f"email-warning account {name} {pwd}")
        return self

    def set_ddns(self,
                    service: str = None,
                    name: str = None,
                    pwd: str = None,
                    domain: str = None):
        """
        Set the Dynamic Domain Name System (DDNS) service. 

            Args: 
                service (str): The DDNS service to use, defaults to 'no-ip'. 
                name (str): The username for the DDNS service. 
                pwd (str): The password for the DDNS service. 
                domain (str): The domain for the DDNS service. 
                
            Returns: 
                self: Returns itself for chaining purposes.
        """
        if service in ['freedns', '3322', 'dyndns', 'no-ip']:
            self._s.command_expect(f"ip ddns service {service}")
        if name:
            self._s.command_expect(f"ip ddns username {name}")
        if pwd:
            self._s.command_expect(f"ip ddns password {pwd}")
        if domain:
            self._s.command_expect(f"ip ddns domain {domain}")
        return self

    def set_dot1x_radius_server(self,
                                server_idx: int,
                                ip: str = None,
                                port: str = None,
                                key: str = None):
        """
        Set the dot1x radius server. This is a wrapper around the'dot1x radius 1st'or'dot1x radius 2nd'command.
            
            Args:
                server_idx: index of server to set ( 1 or 2 )
                ip: ip address to set ( optional ) defaults to None
                port: port to set ( optional ) defaults to None
                key: shared key to set ( optional ) defaults to None
            
            Returns: 
                self for daisychaining ( self )
        """
        
        # This function is used to generate the dot1x radius 1st server
        if server_idx == 1:
            cmd = "dot1x radius 1st-server"
        elif server_idx == 2:
            cmd = "dot1x radius 2nd-server"
        # expect server ip if ip is not set
        if ip:
            self._s.command_expect(f"{cmd} server-ip {ip}")
        # expect command to be sent to server port
        if port:
            self._s.command_expect(f"{cmd} server-port {port}")
        # expect command shared key if key is not provided
        if key:
            self._s.command_expect(f"{cmd} shared-key {key}")
        return self

    def set_dot1x_local_db(self, name: str, pwd: str):
        """
         Set dot1x local database. This is a shortcut for ` dot1x local - userdb username password `
         
            Args:
                name: Name of the user to set
                pwd: Password of the user to set
            
            Returns: 
                self for daisychaining ( self )
        """
        self._s.command_expect(
            f"dot1x local-userdb username {name} password {pwd}")
        return self

    
    def set_authentication_login_mode(self, auth_mode):
        """
         Select the Authentication login option.

            Args:
                 auth_mode: authentication login option.
                 - local: Authenticated in Local.
                 - radius: Authenticated in radius.
                 - radius local: Authenticated in RADIUS and Local by sequence.
        """
        if auth_mode == 'local':
            cmd = 'auth mode local'
        elif auth_mode == 'radius':
            cmd = 'auth mode radius'
        elif auth_mode == 'radius_local':
            cmd = 'auth mode radius local'
        else:
            raise ValueError(f"input invalid auth_mode: {auth_mode}")

        self._s.command_expect(cmd)
        return self

        
    def set_radius_server(self, server_idx, ip, port, key):
        """
         Set RADIUS server. This is a wrapper around the ` ` auth radius server ` ` command.
         
            Args:
                server_idx: The index of the server to set.
                ip: The IP address of the RADIUS server.
                port: The port of the RADIUS server. If the server is listening on a port other than 6379 this will be ignored.
                key: The key used to authenticate the RADIUS server.
            
            Returns: 
                    self for daisychaining ( self ) 
        """
        self._s.command_expect(
            f"auth radius server {server_idx} {ip} port {port} key {key}")
        return self

    
    def set_radius_server_auth_type(self, auth_type):
        """
         Set radius server auth type to login authentication
             
             Args: 
                 auth_type: RADIUS server auth type
                 - pap
                 - chap
                 - peap-mschapv2
        """
        if auth_type not in ["pap", "chap", "peap-mschapv2"]:
            raise ValueError(f"invalid auth_type: {auth_type}")

        self._s.command_expect(f"auth radius auth-type {auth_type}")
        return self


    def set_port_mirror(self, src: str = None, dst: str = None):
        """
        Configure Port mirror on a device.

            Args:
                src (str): monitorPort ( Port ID. Ex. 1/3, Trk2)
                dst (str): mirrorPort (Port ID. Ex. 1/3, 2/1)
            Returns: 
                self for daisychaining ( self )
        """
        if src:
            self._s.command_expect(f"monitor source interface {src}")
        if dst:
            self._s.command_expect(f"monitor destination interface {dst}")
        return self

    def set_hardware_interface(self, enable: bool = True):
        """
        Enable hardware interface (USB) of DUT.

            Args:
                enable (bool, optional): To decide the hardware interface will be turned-on or not.
        """
        if enable == True:
            self._s.command_expect(f"auto-backup enable")
        else:
            self._s.command_expect(f"no auto-backup enable")
        return self

    def set_l37_rule_status(self, index, status):
        """
        Modify the status of a specified L3/L7 rule.

        Args:
            index (int): The index of the L3/L7 policy rule to modify.
            status (str): The status to set for the L3/L7 policy rule. "enable" to enable the rule, "disable" to disable it.
        """
        self._s.command_expect(f"l3l7-policy {index} {status}")
        return self 

    def set_global_vrrp(self, version: str = "3", status: str = "enable"):
        """
         Sets the VRRP status of the device. This is useful for debugging and to make it easier to use the device in a real - time program
         
         Args:
         	 version: The version of the RTP device
         	 status: The status of the RTP device ( enable|disable )
         
         Returns: 
         	 The self for method chaining ( self. _s )
        """
        # Set vrrp version
        if version:
            self._s.command_expect(f"vrrp version {version}")
        # Enable or disable the vrrp
        if status == "enable":
            self._s.command_expect(f"router vrrp")
        elif status == "disable":
            self._s.command_expect(f"no router vrrp")
        else:
            raise ValueError(f"Input invalid status: {status}")
        return self

    class MulticastRoute:
        """Class to configure Multicast Route.

            Args:
                session (object): An instance of the Session class.
        """

        def __init__(self, session: object):
            self._s = session

        def set_global_multicast_route(self, mode: str = "static"):
            """
            Configure global multicast route.
            
                Args
                    action: global multicast route mode (static or disable).
            """
            if mode == "static":
                cmd = f"ip multicast-routing static"
            elif mode == "disable":
                cmd = "no ip multicast-routing"
            else:
                raise ValueError(f"input invalid mode: {mode}")

            self._s.command_expect(cmd)

        def set_static_multicast_route_rule(self,
                                            action: str = "add",
                                            group_addr: str = None,
                                            src_addr: str = None,
                                            in_itf: str = None,
                                            out_itf: str = None):
            """
            Configure global DHCP server.

            Args
                action: configure static multicast route (add / enable / disable).
                group_addr: multicast group
                src_addr: multicast stream source
                in_itf: Inbound interface
                out_itf: Outbound interface
            """
            if action == "add":
                cmd = f"ip mroute group {group_addr} src {src_addr} in \
                            {in_itf} out {out_itf}"

            elif action == "enable":
                cmd = f"ip mroute group {group_addr} src {src_addr} enable"
            elif action == "disable":
                cmd = f"ip mroute group {group_addr} src {src_addr} disable"
            else:
                raise ValueError(f"input invalid action: {mode}")

            self._s.command_expect(cmd)

    def set_malformed_packet_status(self, status: str):
        """
        Configure Malformed Packets Status on a device.

            Args:
                status (str): To decide the Malformed Packets Drop will be "Enabled" or "Disabled".
        """
        if status == "Enabled":
            self._s.command_expect("firewall malformed")
        elif status == "Disabled":
            self._s.command_expect("no firewall malformed")

    def set_malformed_packet_log(self, status: str, dst: set):
        """
        Configure Malformed Packets Log on a device.

            Args:
                status (str): To decide the Malformed Packets Log will be "Enabled" or "Disabled"
                dst (list): the destination of log, can be "flash", "syslog", or "trap"
        """
        if dst.issubset({"flash", "syslog", "trap"}):
            if status == "Enabled":
                for d in dst:
                    self._s.command_expect(
                        f"firewall malformed logging {d}")
            elif status == "Disabled":
                for d in dst:
                    self._s.command_expect(
                        f"no firewall malformed logging {d}")

    class IgmpSnooping:

        def __init__(self, session: object):
            self._s = session

        def set_igmp_snooping_vlan(self, action: str, vid: str):
            if action == 'enable':
                self._s.command_expect(f"ip igmp-snooping vlan {vid}")
            elif action == 'disable':
                self._s.command_expect(f"no ip igmp-snooping vlan {vid}")
            else:
                raise ValueError(f'input invalid action')
            return self

        def set_igmp_snooping_querier(self, action: str, vid: str):
            if action == 'enable':
                self._s.command_expect(
                    f"ip igmp-snooping querier vlan {vid}")
            elif action == 'disable':
                self._s.command_expect(
                    f"no ip igmp-snooping querier vlan {vid}")
            else:
                raise ValueError(f'input invalid action')
            return self

        def set_igmp_snooping_interval(self, interval: str):
            if 20 <= int(interval) <= 600:
                self._s.command_expect(
                    f"ip igmp-snooping query-interval {interval}")
            else:
                raise ValueError(f'input interval out of range')
            return self

    def set_broadcast_forward_status(self, action: str):
        """
        Configure broadcast forward status on a device.

            Args:
                action (str): To decide the broadcast forward will be "Enabled" or "Disabled".
        
        """
        if action == 'enable':
            self._s.command_expect(f"ip broadcast-forward")
        elif action == 'disable':
            self._s.command_expect(f"no ip broadcast-forward")
        else:
            raise ValueError(f'input invalid action')
        return self
    
    def add_broadcast_forward_rule(self, in_itf: str, out_itf: str, udp_port: str):
        """
        Configure broadcast forward rule on a device.

            Args:
                in_itf (str): Inbound interface
                out_itf (str): Outbound interface
                udp_port (str): UDP port
        
        """
        self._s.command_expect(f"ip broadcast-forward in {in_itf} out {out_itf} udp {udp_port}")
        return self
    
    def set_lldp_status(self, action: str):
        """
        Configure LLDP status on a device.

            Args:
                action (str): To decide the LLDP will be "Enabled" or "Disabled".
        
        """
        if action == 'enable':
            self._s.command_expect(f"lldp enable")
        elif action == 'disable':
            self._s.command_expect(f"no lldp enable")
        else:
            raise ValueError(f'input invalid action')
        return self

    def set_lldp_transmission_freq(self, interval: str):
        """
        Configure LLDP Transmission Frequency on a device.

            Args:
                interval (str): LLDP interval
        
        """
        if 5 <= int(interval) <= 32768:
            self._s.command_expect(f"lldp timer {interval}")
        else:
            raise ValueError(f'input interval out of range')
        return self
        
    class Rstp:
        def __init__(self, session: object):
            """
             Initialize the instance. This is called by __init__ and should not be called directly
             
             Args:
             	 session: The session to use
            """
            self._s = session
        def set_rstp_status(self, action):
            """
             Enable or disable RSTP. Redundancy mode can be enabled or disabled at the same time
             
             Args:
             	 action: " enable " or " disable
            """
            # enable disable or rstp.
            if action == "enable":
                self._s.command_expect('redundancy mode rstp')
            # either "spanning tree" or "turbo ring" must be enabled and cannot be enabled or disabled at the same time
            elif action == "disable":
                self._s.command_expect('redundancy mode turbo-ring-v2')
            else:
                raise ValueError(f"input invalid action: {action}")
        
        def set_rstp_port(self, 
                             port,
                             action: str = "enable", 
                             edge_port: bool = False, 
                             priority: str = None, 
                             cost: str = None
                             ):
            """
            Set RSTP settings for a port. This is useful for port selection and retransmission of packets.
            
                Args:
                    port: port to set RSTP settings for.
                    edge_port: if True set RSTP edge settings for this port
                    priority: priority of the packet ( default : 0 )
                    cost: cost of the packet ( default : None )
                
                Returns: 
                    ` ` True ` ` if successful ` ` False ` `
            """
            self._s.command_expect(f'interface ethernet 1/{port}')
            if action == "enable":
                self._s.command_expect(f'spanning-tree')
            elif action == "disable":
                self._s.command_expect(f'no spanning-tree')
            else:
                raise ValueError(f"input invalid status: {action}")
            # expects panning tree edge port command
            if edge_port:
                self._s.command_expect(f'spanning-tree edge-port')
            # expect priority priority if priority is set
            if priority:
                self._s.command_expect(f'spanning-tree priority {priority}')
            # expect the panning tree cost
            if cost:
                self._s.command_expect(f'spanning-tree cost {cost}')
            return self

    def set_static_multicast_table(self, mac: str, port: str = None, vlan: str = None):
        """
        Configure static multicast table on a device

            Args:
                mac (str): Source MAC address.
                port (str): The port use for multicast group. Defaults to None.
                vlan (str): VLAN ID should belong to all of the selected ports.
        """
        self._s.command_expect(f'ip igmp static-group {mac} interface 1/{port} vlan {vlan}')
        return self


    class L2tp:
        def __init__(self, session: object):
            """
             Initialize the instance. This is called by __init__ and should not be called directly
             
             Args:
             	 session: The session to use
            """
            self._s = session

        def set_l2tp_rule(self, local_ip: str, start_ip: str, end_ip: str):
            """
            Configure l2tp interface setting on a device.

                Args:
                    local_ip (str): l2tp server ip.
                    start_ip (str): Starting IP of the Offer IP.
                    end_ip (str): Ending IP of the Offer IP.
            """
            cmd = 'l2tp interface WAN '
            cmd += f'local-ip {local_ip} '
            cmd += f'offer-ip {start_ip} {end_ip}'
            
            self._s.command_expect(cmd)
            return self
        
        def set_l2tp_user(self, username: str, password: str):
            """
            Configure l2tp user setting on a device.

                Args:
                    username (str): The account used for L2TP client login.
                    password (str): The password used for L2TP client login.
            """
            cmd = 'l2tp '
            cmd += f'user {username} '
            cmd += f'password {password}'
            
            self._s.command_expect(cmd)
            return self

    class Snmp:
        def __init__(self, session: object):
            """
             Initialize the instance. This is called by __init__ and should not be called directly
             
             Args:
             	 session: The session to use
            """
            self._s = session
        def set_snmp_version(self, snmp_version: str = None):
            """
            Set the SNMP version.
             
             Args:
             	 snmp_version (str): The 4 SNMP version categories are (v1, v2c, v3) or (v1, v2c) and (v3 only) or (disable).
                                     Default is disable.
            """
            self._s.command_expect(f'snmp-server version {snmp_version}')
            return self
        
        def set_snmp_user(self, 
                            user: str, 
                            priv_key: str,
                            auth_type: str = 'no-auth',
                            priv_method: str = None):
            """
            Set the SNMP version.
             
             Args:
             	 user (str): SNMP username.
             	 priv_key (str): SNMPv3 privacy key.
             	 auth_type (str): SNMPv3 authentication type. Options: 'no-auth', 'md5', 'sha'. Default is 'no-auth'.
             	 priv_method (str): SNMPv3 privacy protocol. Options: 'des', 'aes'. Default is None.
            """
            cmd = f'snmp-server user {user} auth {auth_type} '
            
            if priv_method:
                cmd += f'priv {priv_method} {priv_key}'

            self._s.command_expect(cmd)
            return self

        def set_engine_id(self, engine_id: str):
            """
            Set the SNMP version.
             
             Args:
             	 engine_id (str): Engine id of SNMP agent. Prefix should be 800021f305.
            """
            self._s.command_expect(f'snmp-server engineid {engine_id}')
            return self
        
        def set_snmp_trap_host(self, host: str, community: str = 'public'):
            """
            Set the SNMP host to receive trap/inform notification.
             
            Args:
                host (str): SNMP Host IP Address.
                community (str): SNMP Host community. Default is 'public'.
            """
            self._s.command_expect(f'snmp-server host {host} {community}')
            return self
    
        def set_snmp_trap_inform_v3(self, 
                                    trap_mode: str,
                                    user: str, 
                                    passwd: str, 
                                    auth_type: str = 'no-auth', 
                                    priv_key: str = None ):
            """
            Set the SNMP host to receive trap/inform notification.
             
            Args:
                mode (str): SNMP v3 trap/ inform. Options: 'trap-v3', 'inform-v3'.
                user (str): SNMP trap user.
                passwd (str): Password for authentication.
                auth_type (str): SNMPv3 authentication type. Options: 'no-auth', 'md5', 'sha'. Default is 'no-auth'.
                priv_key (str): SNMP Host community. Default is None.
            """
            cmd = f'snmp-server {trap_mode} user {user} auth {auth_type} {passwd} '

            if priv_key:
                cmd += f'priv {priv_key}'

            self._s.command_expect(cmd)
            return self
                    
        def set_snmp_trap_mode(self, 
                               trap_mode: str, 
                               retry: str = None,
                               timeout: str = None):
            """
            Set the SNMP trap mode.
             
            Args:
                trap_mode (str): SNMP trap mode. Option: trap-v1, trap-v2c, trap-v3, inform (inform-v2c)
            """
            cmd = f'snmp-server trap-mode {trap_mode}'

            if 'inform' in trap_mode:
                cmd += f' retry {retry} timeout {timeout}'

            self._s.command_expect(cmd)
            return self
        



from mdc.router.cli.rp_base.lg_router.edr800 import cli, main, config, config_brg,\
    config_if_lan, config_if_wan, config_if_vlan, config_if_ether

class Base:

    class Cli(cli.Cli):
        pass
    class Main(main.Main):
        pass
    class Config(config.Config):
        pass
    class ConfigIfLan(config_if_lan.ConfigIfLan):
        pass
    class ConfigIfWan(config_if_wan.ConfigIfWan):
        pass
    class ConfigIfVlan(config_if_vlan.ConfigIfVlan):
        pass
    class ConfigIfEthernet(config_if_ether.ConfigIfEthernet):
        pass
    class ConfigBrg(config_brg.ConfigBrg):
        pass


import time
import re
from behave import *
import os


@when('{action} LLDP on "{device}"')
def step_impl(context, action, device):
    """
    action: enable/disable
    """
    context.dut[device].go_config().set_lldp_status(action=action)


@when('set transmission freqency of LLDP updates to {trans_freq} sec on "{device}"')
def step_impl(context, trans_freq, device):
    context.dut[device].go_config().set_lldp_transmission_freq(trans_freq)
    context.lldp_ttl = int(trans_freq)
    context.dut[device].main().save()


@when('starts tshark sniffer on "{host}" for LLDP packet')
def step_impl(context, host):
    context.hosts[host].tshark.start(interface=context.host_info[host]['nic'],
                                     display_filter="lldp",
                                     capture_number=1,
                                     verbose=True)


@then('"{host}" {result} receive LLDP packet from "{device}" within {wait_time} sec')
def step_impl(context, host, result, device, wait_time):
    time.sleep(int(wait_time))
    context.hosts[host].tshark.stop()
    ret = context.hosts[host].tshark.retrieve()
    # print("ret1: ", ret, "end")
    ret = context.hosts[host].tshark.retrieve()
    # print("ret2: ", ret, "end")

    check_packets_captured_pattern = r"Frame \d+:"
    chassis_pattern = r"Chassis Subtype = MAC address, Id: (\S+)"
    port_pattern = r"Port Subtype = Interface name, Id: (\d+)"
    ttl_pattern = r"Time To Live = (\d+) sec"

    if result == "shall not":
        assert bool(re.search(check_packets_captured_pattern, ret)) == False, 'lldp packet captured'

    elif result == "shall":
        assert bool(re.search(check_packets_captured_pattern, ret)), 'no lldp packet captured'

        chassis_match = re.search(chassis_pattern, ret)
        if chassis_match:
            print(chassis_match.group(1))
            assert chassis_match.group(1).lower() == context.dut_info[device]["mac_address"].lower(), \
                f'the lldp packet chassis id: {chassis_match.group(1)} is not the same as the DUT mac address: {context.dut_info[device]["mac_address"]}'

        port_match = re.search(port_pattern, ret)
        if port_match:
            assert port_match.group(1) == context.host_info[host]["testbed"]["port"], \
                f'the lldp packet port id: {port_match.group(1)} is not the same as the host port: {context.host_info[host]["testbed"]["port"]}'

        ttl_match = re.search(ttl_pattern, ret)
        if ttl_match:
            assert int(ttl_match.group(1)) == context.lldp_ttl*4, \
                f'the lldp packet Time To Live: {ttl_match.group(1)} is not the same as the (DUT transmission interval)*4: {context.lldp_ttl}'


@when('"{host}" send {count} LLDP packet with extentions and following values')
def step_impl(context, host, count):
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host]['session']['credential']['password'],
                                                            "../lib/replay_pkts/lldp_with_extention.pcapng",
                                                            context.host_info[host]['session']['credential']['username'],
                                                            context.host_info[host]['session']['host']))

    context.hosts[host].tcp_tool.rewrite(infile="lldp_with_extention.pcapng", outfile="lldp_out.pcap")
    context.hosts[host].tcp_tool.send(sendfile="lldp_out.pcap", number=count)
    time.sleep(5)
    context.lldp_dut_port = context.host_info[host]["testbed"]["port"]
    context.lldp_neighbor_id = context.table[0]['Chasis_id']
    context.lldp_neighbor_port = context.table[0]['Port_id']


@then('the LLDP table on "{device}" should contain an entry with corresponding port, neighbor id, and neighbor port')
def step_impl(context, device):
    ret = context.dut[device].main().show_lldp_table()
    print(ret)
    found_item = None
    for item in ret:
        if item["port"] == context.lldp_dut_port and \
            item["neighbor_id"] == context.lldp_neighbor_id and \
            item["neighbor_port"] == context.lldp_neighbor_port:
            found_item = item
            break

    if found_item:
        print("Found the packet")
    else:
        assert False, f"Packet not found. The table is as following: {ret}"


import time
from host import *
from behave import *
from steps import login, interface, common

@fixture
def run_mail_server(context, host):
    try:
        # -- SETUP-FIXTURE
        context.hosts[host].mail_server.start()
        yield
    finally:
        # -- CLEANUP-FIXTURE
         context.hosts[host].mail_server.clean_up("mail_server")

@fixture
def run_mutt_client(context, host, ip):
    try:
        # -- SETUP-FIXTURE
        context.hosts[host].mail_server.set_mutt_service(ip=ip)
        yield
    finally:
        # -- CLEANUP-FIXTURE
         context.hosts[host].mail_server.clean_up("mutt")

@Given('disable auto-logout on "{device}"')
def step_impl(context, device):
    context.dut[device].go_config()._s.command_expect('ip auto-logout 0')

@When('run email server on "{host}" with following configuration')
def step_impl(context, host):
    use_fixture(run_mail_server, context, host)
    for row in context.table:
        context.hosts[host].mail_server.add_mailserver_account(user=row["user"], paswd=row["paswd"])
    context.hosts[host].mail_server.set_mailserver()
    context.hosts[host].mail_server.set_local_hosts(ip=context.host_info[host]['testbed']['lan_host'])

@When('run mutt client on "{host}" with following configuration')
def step_impl(context, host):  
    use_fixture(run_mutt_client, context, host, ip=context.host_info[host]['testbed']['lan_host'])

@When('set email settings on "{device}" with following configuration')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.MailServer(cfg.get_session()).set_server(context.host_info[context.table[0]["server"]]['testbed']['lan_host'], port='587')
    cfg.MailServer(cfg.get_session()).set_mail_address(context.table[0]["mail_address"])
    cfg.MailServer(cfg.get_session()).set_account(context.table[0]["account"], context.table[0]["paswd"])
    cfg.MailServer(cfg.get_session()).set_sender(context.table[0]["sender"])

@When('send test email from "{device}" to mutt client')
def step_impl(context, device):
    cfg = context.dut[device].go_config()
    cfg.MailServer(cfg.get_session()).send_test()
    time.sleep(20)

@Then('test email can be received by mutt client on "{host}"')
def step_impl(context, host):
    ret = context.hosts[host].mail_server.get_email()
    assert "MOXA Test Mail" in ret, f'Test mail not received: {ret}'


from mdc.router.cli.rp_base.ng_router import config
from mdc.router.cli.rp_base.ng_router.cli import PROMPT_CONFIG

class ConfigIpsecPhase1(config.Config):
    """
    Class for configuring IPSecPhase1.

        Args:
            session (object): A valid session object.
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Attributes:
            _s (Session): A Session object used to communicate with the device.
            _args (list): A list of arguments passed to the function.
            _kwargs (dict): A dictionary of keyword arguments passed to the function.
    """

    def __init__(self, session: object, *args, **kwargs):
        # Auxiliary method under ConfigIPSec so super init is not required
        self._s = session
        self._args = args
        self._kwargs = kwargs

    def _set_default_prompts(self):
        """
        Set the default prompts.
        """
        self._s.set_default_prompts([PROMPT_CONFIG])

    def _getin(self):
        """
        Get into the interface lan.
        """
        self._s.command_expect(f'phase1')

    def getin(self):
        """
        Set the default prompts and get into the interface wan.

            Returns: 
                self: The instance of the object. 
        """
        self._set_default_prompts()
        self._getin()
        return self

    def set_key_exchange(self,
                            ike_mode: str = None,
                            ike_version: str = None,
                            auth_mode_psk: str = None,
                            encryption: str = None,
                            hash_algo: str = None,
                            dh_group: str = None,
                            life_time: str = None):
        if ike_mode:
            self._s.command_expect(f'ike-mode {ike_mode}')
        if ike_version:
            self._s.command_expect(f'ike-version {ike_version}')
        if auth_mode_psk:
            self._s.command_expect(f'auth-mode psk {auth_mode_psk}')
        if encryption:
            self._s.command_expect(f'encryption {encryption}')
        if hash_algo:
            self._s.command_expect(f'hash {hash_algo}')
        if dh_group:
            self._s.command_expect(f'dh-group {dh_group}')
        if life_time:
            self._s.command_expect(f'life-time {life_time}')
        # quit to save
        self._s.command_expect(f'exit')
        return self



import sys
from host import *
from host import HostFactory
from topology.topology_info import prepare_json
from steps.common import login_and_measure_time

def _reset(context):
    context.dut["DUT"] = HostFactory.create_mdc_rp(
    context.dut_info["DUT"], 
    debug_output=sys.stdout)
    context.dut["DUT"].main().reload_factory_default()
    time.sleep(context.dut_info["DUT"]['reload_delay'])
    context.dut["DUT"].close()

def _backup(context):
    login_time = login_and_measure_time(context=context, device="DUT")
    print(f"Reload Factory Time: {context.dut_info['DUT']['reload_delay']} + {login_time:.2f} seconds")
    ret = context.dut["DUT"].main().export_config(
        tftp_ip=context.host_info["HOST_EXECUTOR"]["testbed"]["cur_host"], 
        file_name="default.ini")
    assert ret['matched'] and '^Parse error' not in ret[
        'data'], f'Fail to prepare default config: {ret["data"]}'
    context.dut["DUT"].close()

def after_step(context, step):
    """
    In order to prevent traffic conflicts, internet port will be enabled for the DDNS case and disabled for all other cases.

        Args:
            step: specific step
    """
    
    if step.name == 'reload factory-default "DUT"':
        port = context.dut_info["DUT"]["testbed"]["internet_port"]
        flag = False
        tags = context.scenario.tags
        for t in tags:
            if 'ddns' in t or 'ping' in t:
                flag = True
        if not flag:
            context.dut["DUT"].go_config_if_ethernet(f'1/{port}').set_port_status(flag = 'disable')
        else:
            context.dut["DUT"].go_config_if_ethernet(f'1/{port}').set_port_status(flag = 'enable')

def before_all(context):
    prepare_json(context)
    # do reload factory to prepare default configuration for initializing with restoration
    if context.dut_info["DUT"]["init_method"] == "restore":
        _reset(context)
        _backup(context)


def before_scenario(context, scenario):
    prepare_json(context)


def after_scenario(context, scenario):
    pass


def after_feature(context, feature):
    if context.dut_info["DUT"]["init_method"] == "restore":
        print("reset to disable config encrypt setting for restoration")
        reset_list = ["config_encrypt", "user_account", "user_interface"]
        if any(x in feature.filename for x in reset_list):
            _reset(context)


from host import *
from behave import *
from steps.login import loginToDUT
from steps import common, interface, login


@given(u'import max firewall rule configuration from "{host}" to "{device}"')
def step_impl(context, device, host):
    context.dut[device].main()._s.command_expect(
        command=
        f'copy tftp {context.host_info[host]["testbed"]["cur_host"]} config-file max_firewall.ini',
        prompts=['Config file import successfully.'],
        timeout=60)
    context.dut[device].close()
    loginToDUT(context, device)


@given(u'set virtual interfaces on "{device}" ports')
def step_impl(context, device):
    for row in context.table:
        if row['virtual interfaces'] == 'WAN':
            context.dut[device].go_config().set_vlan_id(
                context.dut_info[device]["testbed"]["wan_vlan"])
            context.dut[device].go_config_if_ethernet(
                f'1/{row["ports"]}').set_switchport(
                    context.dut_info[device]["testbed"]["wan_vlan"])
            context.dut[device].go_config_if_wan().set_wan_interface(
                ip=row["virtual ip"],
                mask="255.255.255.0",
                bind_vlan=row["ports"])
        elif row['virtual interfaces'] == 'BRG':
            context.dut[device].go_config_if_ethernet(f'1/{row["ports"]}'). \
                                set_switchport(context.dut_info[device]["testbed"]["brg_vlan"]) .\
                                set_bridge_group()
            context.dut[device].go_config_bridge().set_brg_interface(
                ip=row["virtual ip"], mask="255.255.255.0")
        else:
            raise ValueError(
                f'input virtual interfaces:{row["virtual interfaces"] } is not exist'
            )


@given(
    u'cycle a maximum number of rules with a set of firewall rules and only sepcified index rule mapping the traffic on "{device}"'
)
def step_impl(context, device):
    '''
        Firewall rule list will be like:
        index 0 deny_ICMP_0 from WAN (128.94)      to LAN (127.93)
        index 1 deny_TCP_1  from LAN (dismatch_ip) to WAN (128.93)
        index 2 deny_UDP_2  from BRG (dismatch_ip) to BRG (126.93)
        .
        .
        .
        index 511 deny_ICMP_511 from WAN (dismatch_ip) to LAN (128.93)
        index 512 deny_TCP_512  from LAN (127.94)      to WAN (128.93)
        .
        .
        .
        index 1022 deny_TCP_1022 from LAN (dismatch_ip) to WAN (128.93)
        index 1023 deny_UDP_1023 from BRG (126.94)      to BRG (126.93)
    '''
    src_ip, dst_ip = {}, {}
    specific_index, protocol, incoming_interface, outgoing_interface, sender, receiver = [], [], [], [], [], []

    # store necessary information for creating the firewall rule
    for row in context.table:
        specific_index.append(row['specified index'])
        protocol.append(row['specific protocols'])
        incoming_interface.append(row['incoming interface'])
        outgoing_interface.append(row['outgoing interface'])
        sender.append(row['sender'])
        receiver.append(row['receiver'])
    print(specific_index, protocol, incoming_interface, outgoing_interface)

    # loop for createing object
    for i in range(3):
        src_ip[incoming_interface[i]] = context.host_info[
            sender[i]]["testbed"][f"{incoming_interface[i].lower()}_host"]
        context.dut[device].go_config_object_addr().create_object(name=f"src_{incoming_interface[i].lower()}", \
                                                                  ip_addr=src_ip[incoming_interface[i]])

        dst_ip[outgoing_interface[i]] = context.host_info[
            receiver[i]]["testbed"][f"{outgoing_interface[i].lower()}_host"]
        context.dut[device].go_config_object_addr().create_object(name=f"dst_{outgoing_interface[i].lower()}", \
                                                                  ip_addr=dst_ip[outgoing_interface[i]])

    context.dut[device].go_config_object_addr().create_object(name=f"dismatch_10", \
                                                              ip_addr=f"10.10.10.99")
    # loop for creating service object
    for p in protocol:
        context.dut[device].go_config_object_serv(p.lower()). \
        command(f"name {p.upper()}")
    context.dut[device].main().command('show object')

    # loop for creating firewall rule
    for i in range(1024):
        if f'{i}' in specific_index:
            index = specific_index.index(f'{i}')
            context.dut[device].go_config().command(f"logging l3l7-policy")
            context.dut[device].go_config_l37policy().set_l37_policy(
                name=f"deny{protocol[index]}_{i}",
                enable=True,
                logging="enable",
                itf={
                    'in': incoming_interface[index].upper(),
                    'out': outgoing_interface[index].upper()
                },
                policy_action="deny",
                mode='ip',
                sip=f"src_{incoming_interface[index].lower()}",
                dip=f"dst_{outgoing_interface[index].lower()}",
                dport=protocol[index].upper())
        else:
            context.dut[device].go_config().command(f"logging l3l7-policy")
            context.dut[device].go_config_l37policy().set_l37_policy(
                name=f"deny{protocol[i%3]}_{i}",
                enable=True,
                logging="enable",
                itf={
                    'in': incoming_interface[i % 3].upper(),
                    'out': outgoing_interface[i % 3].upper()
                },
                policy_action="deny",
                mode='ip',
                sip=f"dismatch_10",
                dip=f"dst_{outgoing_interface[i%3].lower()}",
                dport=protocol[i % 3].upper())
    context.dut[device].main().command('show l3l7-policy')



import time
import random

from behave import *
from mdc.router.cli.rp_base.ng_router.base import Base
from mdc.router.cli.rp_base.lg_router.tn5000.base import Base as tn5000
from steps import common, interface, login

@when(u'send {pkt_count} vrrp packet with the content of the table from "{host}" to "{device}"')
def step_impl(context, pkt_count, host, device):
    ip_address = context.table[0]["addr_list"]
    padding_n = 14
    script = context.hosts[host].scapy.generate_vrrp_scapy_script(padding_n=padding_n,
                                                                  smac=context.host_info[host]["testbed"]["mac_address"],
                                                                  dmac="01:00:5e:00:00:12",
                                                                  sip=context.host_info[host]["testbed"]["cur_host"],
                                                                  dip="224.0.0.18",
                                                                  version=3,
                                                                  priority=context.table[0]["priority"],
                                                                  ip_address=ip_address,
                                                                  iface=context.host_info[host]["nic"]
                                                                  )
    from datetime import datetime
    import os

    # write the scapy script to file
    scapy_file_name = datetime.now().isoformat().replace(":", "_")[:19]
    with open(f'../lib/atb/service/scapy_template/{scapy_file_name}', "w") as script_file:
        script_content = script_file.write(script)
    
    os.system('sshpass -p "{0}" scp {1} {2}@{3}:~/'.format(context.host_info[host]['session']['credential']['password'],
                                                            f"../lib/atb/service/scapy_template/{scapy_file_name}",
                                                            context.host_info[host]['session']['credential']['username'],
                                                            context.host_info[host]['session']['host']))
    # Clean up
    for _ in range(int(pkt_count)):
        context.hosts[host].scapy.do_script(scapy_file_name)
    os.system(f'rm ../lib/atb/service/scapy_template/{scapy_file_name}')

@when(u'set up vrrp on the "{device}" with the content of the table')
def step_imple(context, device):
    version = context.table[0]["version"]
    status = context.table[0]["status"]
    vrid = context.table[0]["vrid"]
    vip = context.table[0]["vip"]
    priority = context.table[0]["priority"]
    preempt_mode = context.table[0]["preempt_mode"]
    preempt_delay = context.table[0]["preempt_delay"]
    accept_mode = context.table[0]["accept_mode"]
    interface = context.table[0]["interface"]
    adver_interval = context.table[0]["adver_interval"]
    if issubclass(context.dut[device]._model, tn5000):
        context.adver_interval = int(adver_interval) / 100
    else:
        context.adver_interval = int(adver_interval) / 1000
    context.dut[device].go_config_vrrp(1).set_vrrp_rule(status = status,
                                                     vrid = vrid,
                                                     vip = vip,
                                                     priority = priority,
                                                     preempt_mode = preempt_mode,
                                                     preempt_delay = preempt_delay,
                                                     accept_mode = accept_mode,
                                                     interface = interface,
                                                     adver_interval = adver_interval,
                                                     adver_interval_ver = version
                                                     )
    context.dut[device].go_config().set_global_vrrp(version = version, status = status)


@then(u'vrrp status should be "{status}" on "{device}"')
def step_imple(context, status, device):
    flag = False
    for _ in range(int(context.adver_interval)):
        ret = context.dut[device].main().show_vrrp()
        if status in ret[0]["vrrp_status"]:
            flag = True
            break
        time.sleep(1)
    assert flag, f'vrrp do not change -> expect: {status} but {ret[0]["vrrp_status"]}'

@when(u'wait for {num} advertisement interval')
def step_imple(context, num):
    time.sleep(int(num) * int(context.adver_interval))


from .hping3 import Hping3Service
from .pktmgmt import PktMgmtService
from .scapy import ScapyService
from .shellcmd import ShellCmdService
from .syslog import SyslogService
from .tftp import TftpService
from .trex import TrexService, TrexServerService, TrexConsoleService
from .tshark import TsharkService
from .uictrl import UICtrlService
from .ca import CaService
from .bird import BirdService
from .nmcli import NetworkManagerService
from .radius import RadiusService
from .snmp import SnmpService
from .mail_server import MailServerService



from scapy.all import *
import random
source_macs = {source_macs}
ips = {ips}
for source_mac, ip in zip(source_macs, ips):
    arp_request = Ether(src=source_mac, dst="ff:ff:ff:ff:ff:ff") / ARP(psrc=ip, pdst="{target_ip}")
    sendp(arp_request, iface='{nic}', count=1)


import re
import sys
import time

from host import *
from behave import *
from steps import common, interface, login


def check_message(ret, msg):
    for r in ret:
        if msg in r['event']:
            return True
    return False

@given(u'run tftp service on "{host}"')
def step_impl(context, host):
    context.hosts[host].tftp.check_env()

@given(u'get running firmware version from "{device}"')
def step_impl(context, device):
    ret = context.dut[device].main().show_version()
    model_tmp = ret[0]['model_name'].split('-')
    model = model_tmp[0] + '-' + model_tmp[1]
    version = ret[0]['firmware_version'].replace("build", "Build").replace(" ", "_") 
    if 'TN-49' in model:
        model = 'TN-4900'
    elif 'TN-59' in model:
        model = model.replace("-", "")
    context.fw = 'FWR_' + model + '_' + version + '.rom'
    print(context.fw)

@then(
    u'"{device}" record two event logs "{message_1}" and "{message_2}" with "Firmware Upgrade"'
)
def step_impl(context, device, message_1, message_2):
    ret = context.dut[device].main().show_logging_event_log_system()
    assert check_message(ret, message_1) and check_message(ret, message_2), \
    f"DUT fail to record event logs: expect {message_1} and {message_2}, but {[r['event'] for r in ret]}"

@then(u'"{device}" record one event log "{message_1}"')
def step_impl(context, device, message_1):
    ret = context.dut[device].main().show_logging_event_log_system()
    assert message_1 in [r['event'] for r in ret], \
    f"DUT fail to record event logs: expect {message_1}, but {[r['event'] for r in ret]}"

@then(u'show version containing the correct information of {firmware} on "{device}"')
def step_impl(context, device, firmware):
    version = re.findall(r'V(.*)_Build_(\d+)', firmware)[0][0]
    build = re.findall(r'V(.*)_Build_(\d+)', firmware)[0][1]
    ret = context.dut[device].main().show_version()
    assert version in ret[0]['firmware_version'] and build in ret[0]['firmware_version'], \
    f"Expect {version}_{build} but {ret[0]['firmware_version']}"

@then(u'check "{device}" is upgraded back to the running firmware')
def step_impl(context, device):  
    version = re.findall(r'V(.*)_Build_(\d+)', context.fw)[0][0]
    build = re.findall(r'V(.*)_Build_(\d+)', context.fw)[0][1]
    ret = context.dut[device].main().show_version()
    assert version in ret[0]['firmware_version'] and build in ret[0]['firmware_version'], \
    f"Expect {version}_{build} but {ret[0]['firmware_version']}"

@when(u'upgrade "{device}" back to running firmware')
def step_impl(context, device):  
    ret = context.dut[device].main().upgrade_firmware(
        tftp_ip=context.host_info["HOST_EXECUTOR"]["testbed"]["cur_host"],
        file_name=context.fw)
    context.dut[device].close()
    assert ret['matched'] and '^Parse error' not in ret['data']
    login.loginToDUT(context, device)
    time.sleep(10)



from session import SshSession
from utils.decorator import root_access_required

class PktMgmtService:
    """
    Class for manipulating TCP traffic.

        Args:
            ssh_session (SshSession): An SSH session object.
            sudo_paswd (str): The sudo password for the remote server.

        Attributes:
            _ssh (SshSession): SSH session.
            _sudo_pwd (str): Sudo password.
    """

    def __init__(self, ssh_session: SshSession, sudo_paswd: str):
        self._ssh = ssh_session
        self._sudo_pwd = sudo_paswd

    def _check_conneted(self):
        """
        Check if SSH session is connected.

        Raises:
            ValueError: If SSH session is not connected.
        """
        if not self._ssh.is_connected():
            raise ValueError('SSH session not connected')

    def check_env(self):
        """
        Check if the environment have tcpreplay.

        Raises:
            ValueError: If the Command 'tcpreplay' is not found.
        """
        self._check_conneted()
        output = self._ssh.command('tcpreplay -V')
        if 'not found' in output:
            raise ValueError('Command "tcpreplay" not found')

    def rewrite(self,
                infile: str = "input.pcap",
                outfile: str = "output.pcap",
                sip: str = None,
                dip: str = None,
                smac: str = None,
                dmac: str = None):
        """
        Rewrite a given pcap file and send it through a given interface.

            Args:
                infile (str, optional): Path to the input pcap file. Defaults to None.
                outfile (str, optional): Path to the output pcap file. Defaults to None.
                sip (str, optional): Source IP address for rewriting. Defaults to None.
                dip (str, optional): Destination IP address for rewriting. Defaults to None.
                smac (str, optional): Source MAC address for rewriting. Defaults to None.
                dmac (str, optional): Destination MAC address for rewriting. Defaults to None. 
                intf (str, optional): Interface used for sending the pcap file through tcpreplay command . Defaults to None. 
                sendfile (str, optional): Path of the pcap file that needs to be sent through tcpreplay command . Defaults to None. 
                number (int, optional): Number of packets that needs to be sent through tcpreplay command . Defaults to None. 

            Returns: 
                ret(string) : Output of tshark command after sending the packet through tcpreplay command  

            Raises: 
                ValueError: If SSH session is not connected
        """
        self._check_conneted()

        cmd = 'tcprewrite '
        cmd += f'--infile={infile} '
        cmd += f'--outfile={outfile} '
        cmd += '' if sip is None else f'--srcipmap=0.0.0.0/0:{sip} '
        cmd += '' if dip is None else f'--dstipmap=0.0.0.0/0:{dip} '
        cmd += '' if smac is None else f'--enet-smac={smac} '
        cmd += '' if dmac is None else f'--enet-dmac={dmac} '
        cmd += '--fixcsum'

        print(cmd)

        self._ssh.command(cmd)

        capture_name = outfile.split('/')[-1].split('.')[0]
        self._ssh.command(f'tshark -r {outfile} -V > {capture_name}.log')

        ret = self._ssh.command(f'cat {capture_name}.log')
        print(ret)

        return ret

    @root_access_required
    def send(self, intf: str = None, sendfile: str = None, number: int = None):
        self._check_conneted()
        """
        Send a packet capture file to the target device. 
            
            Args: 
                intf (str): Interface to use for sending the packet capture file. Defaults to None. 
                sendfile (str): Path of the packet capture file to send. Defaults to None. 
                number (int): Number of packets to send. Defaults to None.  

            Raises: 
                Exception: If connection is not established or login as root fails.
        """
        # cmd = 'docker run --rm -t -v $(pwd):/data -i dgarros/tcpreplay /usr/bin/tcpreplay '
        cmd = 'tcpreplay '
        cmd += f'--intf1=eth1 ' if intf is None else f'--intf1={intf} '
        cmd += f'' if number is None else f'-l {number} '
        cmd += f'./home/autotwo/output.pcap ' if sendfile is None else f'{sendfile} '

        print(cmd)

        self._ssh.command(cmd)
